/*-----------------------------------------------------------
 * File:    imb_lin.c
 * Project: IMB Driver for Linux 
 *----------------------------------------------------------
 * Copyright (c) 1999-2006 by Intel Corporation
 *---------------------------------------------------------*/
/*M*                     
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
 *M*/
/*---------------------------------------------------------*/
/*M*
//  PVCS:
//      $Workfile:   imb_lin.c  $
//      $Revision: 1.2 $
//      $Modtime:   06 Aug 2001 13:12:48  $
//
//		$Log: imb_lin.c $
//		Revision 1.2  2003/04/01 14:25:26Z  marcela
//		add semaphore
//		Revision 1.1  2002/12/11 06:12:04Z  mks
//		Initial revision
//		Revision 1.5  2002/12/10 12:24:36Z  marcela
//		Change OS timer func names
//		Revision 1.4  2002/12/09 12:48:56Z  marcela
//		Add the module license macro.
//		Replace the obsolete linux71 flag by something more dynamic!
//		Revision 1.3  2002/12/01 15:24:38Z  marcela
//		Add SMBus
//		Revision 1.2  2002/11/12 13:56:54Z  marcela
//		Revision 1.1  2002/07/07 03:14:00Z  mks
//		Initial revision
 * 
 *   Build 16  14 Nov 2006 13:45:16   Andy Cress 
 * Handle CONFIG_COMPAT for Linux kernels >= 2.6.14
 *   Build 15  25 Aug 2006 13:45:16   Andy Cress 
 * Handle 32-bit apps in 64-bit driver via CONFIG_COMPAT 
 *   Build 14  28 Jul 2006 12:00:00   Andy Cress 
 * Handle remap_pfn_range if Linux >= 2.6.10  
 *   Build 13  13 Jul 2006 12:00:00   Andy Cress 
 * Add S5000PHB to getboard for TIGW1U in spec file
 *   Build 12  22 Jun 2006 12:00:00   Andy Cress 
 * Allow lsmod in /usr/bin, for MVL CGE 4.01 et al
 *   Build 11  02 Jun 2006 12:00:00   Andy Cress 
 * Reuse addr in SmbCharact, equiv to Win imbdrv.sys v4.4
 *   Build 10  02 May 2006 12:00:00   Andy Cress 
 * Reuse probe outcome after first time, x86_64 debug
 *   Build 9   14 Apr 2006 15:45:30   Andy Cress 
 * Added module parameter for spacing 
 *    Rev 1.8   06 Aug 2001 13:14:46   spoola
 * Fixed tracker items #15667, #15666, #15664
 *    Rev 1.7   01 Aug 2001 14:24:58   spoola
 * Added comments in reference to #15667, #15666, #15664
 *    Rev 1.6   24 Jul 2001 16:25:08   spoola
 * Fixed tracker item #15462
 *    Rev 1.5   23 Jul 2001 11:18:10   spoola
 * Fixed tracker item #15462
 *    Rev 1.4   20 Jul 2001 13:48:46   spoola
 * Fixed tracker item #15461
 *    Rev 1.3   27 Jun 2001 08:43:22   spoola
 * Fixes made for linux7.1
 *    Rev 1.2   31 May 2001 15:51:58   MRAMACHA
 * Modifications done to handle TSD's Telco alarm app along with basebrd (ie, mulitple apps accessing the driver simultaneously).
 *    Rev 1.1   May 23 2000 16:39:42   simont
 * 2.3 to 3.0 code merge.
 *    Rev 1.3   14 Oct 1999 12:58:28   mramacha
 * Changes to mmap.
 *    Rev 1.2   09 Sep 1999 13:25:48   mramacha
 * Changes to ioctl and mmap
 *    Rev 1.1   Aug 29 1999 20:38:18   mramachx
 * Code cleanup and buf fixes.
 *    Rev 1.0   Aug 27 1999 16:55:50   mramachx
 * Initial Linux Checkin   (Mahendra Ramachandran)
 *
//
//  Purpose:    This file contains the Linux driver entry points as well as 
//				Linux specific code for the IMB driver.
//
//  Contents:
// 			init_module
// 			cleanup_module
// 			IMB_open
// 			IMB_close
// 			IMB_ioctl
// 			IMB_ioctl32
// 			IMBmmap 
//			DriverEntry
//			DriverExit
//			ImbShutdown
//
//
//  Notes:
//			The driver in Linux (as in Unixware and Solaris) can be opened
//			by only one user-level app since there is only one device entry.
//			Hence we do not worry about maintaining per-device state. But this
//			can be modified fairly easily. MR
//
*M*/

/* Note that linux/module.h must be included only here and not the other
 * files. Hence it is not in imb_os.h. Mahendra
 */
#include <linux/module.h>
#include <linux/notifier.h> 
#include "imb_os.h"
#include "imb_drv.h"
#include "kcs.h"

/* Note that a major of 0 means the OS can dynamically asign a major number 
 * for our device. Our startup script in /etc/rc.d/init.d/isc must ensure
 * that we clean up stale /dev entries and create new ones based on the
 * Major number assigned with our module was loaded.
**/
#define	DEFAULT_IMB_MAJOR 0 
#define IMB_DRV_NAME "imb"

/////////////////////////////////////////////////////////////////
//  prototypes for driver entry points
/////////////////////////////////////////////////////////////////
static int  IMB_open(struct inode *, struct file *);
static int  IMB_release(struct inode * , struct file * );
static int  IMB_ioctl(struct inode *, struct file *, unsigned int , unsigned long );

#ifdef CONFIG_COMPAT
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
/* Linux 2.4 seems to have problems with compat_ptr() */
static inline void *compat_ptr(unsigned int uptr)
{
        return (void *)(unsigned long)uptr;
}
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14))
static long IMB_ioctl32(struct file *filep, unsigned int cmd, 
			unsigned long arg );
#else
static int  IMB_ioctl32(unsigned int fd, unsigned int cmd, 
			unsigned long arg, struct file *filep);
#endif
#endif
static int  IMBmmap(struct file *, struct vm_area_struct *);

////////////////////////////////////////////////////////////////
// other function prototypes
////////////////////////////////////////////////////////////////
static int DriverEntry( PDEVICE_EXTENSION deviceExtension);
static void DriverExit(PDEVICE_EXTENSION deviceExtension);
static NTSTATUS ImbShutdown ( PDEVICE_EXTENSION deviceExtension);
VOID getBmcIpmiVersion( PDEVICE_EXTENSION deviceExtension);
void ImbUnixAsyncDeallocate(void *context);
int  ImbUnixAsyncInit ( PDEVICE_EXTENSION deviceExtension);
void ImbUnixAsyncLock ( PDEVICE_EXTENSION deviceExtension);
extern void TimerDpcFuncM(unsigned long arg);
void Delay(DWORD delayTime); //in usec 10(-6)
void UpdateTimeStampThread(void * arg);
unsigned long GetRtcTime(void);

///////////////////////////////////////////////////////////
// Global state variable
//////////////////////////////////////////////////////////
int imb_major;
kthread_t kthreadStruct; // Update Timestamp Thread
int spacing=1;    // Register Spacing parameter via "load_driver" - Default=1
int smbaddr=0x42;  // Parameter received from "load_driver" - Default=0x42
int _smbaddr=0x84; /* internally used, =smbaddr*2 (0x84=132.) */
int kcsbase = KCS_BASE;   /*KCS_BASE=0xca2  default*/
int smicaddr = 0xCA9;   /*SMIC_DATA_REGISTER=0xCA9  default*/
int debug=0;       /* Debug parameter, default=0 (off) */
char *imb_ver = IMB_VER;
extern int kcs_command_register; /*+1*/
extern int kcs_status_register; /*+1*/
extern int kcs_datain_register;
extern int kcs_dataout_register;

/////////////////////////////////////////////////////////////////////////////
// The file_operations defines the driver entry points we support in imb.
// It is used by init_module() to register the driver. Note that I have
// defined IMB_fops in a way supported by gcc compiler. This way I don't
// have to worry if the struture is changed in future releases.
//
// If you need to compile using a different compiler, you may need to ifdef
// the declaration for gcc and in the else put the complete structure with
// NULLs for the other fields.
/////////////////////////////////////////////////////////////////////////////
// #if LINUX_VERSION_CODE < 0x020600
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
/* Linux 2.4 kernels */
static struct file_operations    IMB_fops = {
    open:	IMB_open,
    release:	IMB_release,
    ioctl:	IMB_ioctl,
    mmap:       IMBmmap
};
MODULE_PARM(smbaddr, "i");
MODULE_PARM(kcsbase, "i");
MODULE_PARM(spacing, "i");
MODULE_PARM(debug, "i");
#else
/* Linux 2.6 kernels */
#include <linux/moduleparam.h>
#include <linux/mm.h> 
#include <linux/slab.h> 
#include <linux/vmalloc.h>
#ifdef CONFIG_COMPAT
/* Add support for 32bit ioctls on 64bit kernel */ 
#include <linux/compat.h>
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22))
#include <linux/ioctl32.h>
#endif
#endif
#ifndef BCD_TO_BIN
#define BCD_TO_BIN(val) ((val)=((val)&15) + ((val)>>4)*10)
#endif
static int use_count = 0;
static struct file_operations    IMB_fops = {
    owner:      THIS_MODULE,   /*required in 2.6*/
    ioctl:      IMB_ioctl,
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,13))
#ifdef CONFIG_COMPAT
    compat_ioctl:  IMB_ioctl32,
#endif
#endif
    open:       IMB_open,
    release:    IMB_release,
    mmap:       IMBmmap
};
module_param(smbaddr, int, 0644);
module_param(kcsbase, int, 0644);
module_param(spacing, int, 0644);
module_param(debug,   int, 0644);
#endif

MODULE_AUTHOR("Andrew Cress, Intel Corp.");
MODULE_DESCRIPTION("Intel IMB IPMI driver to communicate with Management Controller");
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,11))
MODULE_LICENSE("GPL");
#endif

//////////////////////////////////////////////////////////
// A couple of strictly local types
//////////////////////////////////////////////////////////
// structure used when managing ISA property "3-tuples"
typedef struct isa_reg {
    uint32_t    type;
    uint32_t    addr;
    uint32_t    size;
} isa_reg_t;

// Size field used to determine size of table mapping is defined in DMI BIOS
// spec to be a word type, or 16 bits wide.  It's implemented in dmibio.h
// as an unsigned short (DMI_BIOS_ENTRY_POINT_STRUCT.tableLength).  Hence,
// the max is 65535.  Need to pin down a max because you can't get the size
// requested from the args to _mmap().
#define MAX_REQ_MAP_SIZE    65535


//
// These two defines are used to signal success or failure between functions
// They are similar to DDI_SUCCESS and DDI_FAILURE in DDI/DKI compliant
// unixes. The driver entry points will use errno values to signal failure 
// to the OS and user-level code.

#define SUCCESS 0
#define FAILURE -1

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,16))
#define PANIC_LIST_STRUCT struct atomic_notifier_head 
#else
#define PANIC_LIST_STRUCT struct notifier_block *
#endif

int fsmbus = 0;  /* default to kcs */
int ftimestampthread = 0; /* default to off, on causes clock=0 & badness */

extern PANIC_LIST_STRUCT panic_notifier_list;
BmcRequestProc    g_sendBmcRequest = NULL;   /*used in imb_panic_event*/
int imb_has_paniced = 0;

///////////////////////////////////////////////////////////
//   Function Definitions begin Here
///////////////////////////////////////////////////////////

int imb_panic_event( struct notifier_block *this, unsigned long event,
                        void *ptr)
{
    unsigned char reqbuf[MAX_BUFFER_SIZE]; 
    unsigned char respbuf[MAX_BUFFER_SIZE]; 
    BmcRequest * bmcReq = (BmcRequest *)reqbuf;
    DWORD respLength = 0;
    unsigned char *obuf;
    char *panic_str = NULL;
    int rv, rqlen;
    int sz, i, j;

    if (g_sendBmcRequest == NULL) return NOTIFY_DONE;
    if (imb_has_paniced) return NOTIFY_DONE;
    imb_has_paniced = 1;

    bmcReq = (BmcRequest *)reqbuf;
    obuf = &bmcReq->data[0];
    panic_str = (char *)ptr;

    /* insert the panic event in the SEL */
    /* Log the OS Critical Stop to the SEL (BMC firmware log). */  
    obuf[0] = 0x21; /* Kernel generator ID, IPMI table 5-4 */
    obuf[1] = 0x03; /* 3=IPMI10, 4=IPMI15, set as back-compatible w 1.0 */
    obuf[2] = 0x20; /* OS Critical Stop, IPMI table 36-3 */
    obuf[4] = 0x6f; /* Sensor specific, IPMI table 36-1 */
    obuf[5] = 0xa1; /* Runtime stop OEM bytes 2 & 3. */
    if (panic_str != NULL) {
       obuf[3] = panic_str[0];
       obuf[6] = panic_str[1];
       obuf[7] = panic_str[2];
    } else {
       obuf[3] = 'l';
       obuf[6] = 'n';
       obuf[7] = 'x';
    }
    /* Most linux has zero for panic event, but just in case. */
    if (event != 0) obuf[6] |= 0x80; /*set Int! bit if so*/
    bmcReq->nfLn            = (0x04 << 2) + 0x00; // NETFN_LUN SEVT
    bmcReq->cmd             = 0x02;  // PLATFORM_EVENT;
    rqlen = 8 + MIN_BMC_REQUEST_SIZE;
    respLength = sizeof(respbuf);
    rv = g_sendBmcRequest(bmcReq, rqlen, (BmcResponse *)respbuf, &respLength, 
			  NULL, 0);
    printk("imb_panic_event: send event rv = %d\n",rv); 

    /* Now write the full panic string to the SEL also. */
    sz = strlen(panic_str);
    bmcReq->nfLn            = (0x0a << 2) + 0x00; // NETFN_LUN STOR
    bmcReq->cmd             = 0x44;  // ADD_SEL_ENTRY
    rqlen = 16 + MIN_BMC_REQUEST_SIZE;
    j = 0;
    for (i = 0; i < sz;  ) 
    {
	obuf[0] = 0;
	obuf[1] = 0;
	obuf[2] = 0xf0; /*OEM event without timestamp*/
	obuf[3] = BMC_SA; /*BMC addr = 0x20*/
	obuf[4] = j++;  /*sequence number*/
	strncpy(&obuf[5],&panic_str[i],11);
	respLength = sizeof(respbuf);
	rv = g_sendBmcRequest(bmcReq,rqlen,(BmcResponse *)respbuf,&respLength,
			     NULL, 0);
        printk("imb_panic_event: add_sel rv = %d\n",rv);  //++++
	i += 11;
    }

    return NOTIFY_DONE;
}

static struct notifier_block imb_panic_block = {
       imb_panic_event,
       NULL,
       200   /* priority: INT_MAX >= x >= 0 */
}; 

///////////////////////////////////////////////////////////
//  init_module
///////////////////////////////////////////////////////////
/*F*
//  Name:       init_module
//  Purpose:    supports loading of the driver
//  Context:    called by system when loading the driver as a 
//				result of the insmod command
//  Returns:    0 on success, negative number on failure
//  Parameters: none
//  Notes:      none
*F*/
int
init_module(void)
{
       int err = 0;

	//
	//Register with the kernel our loadable module
	//
        imb_major = register_chrdev(DEFAULT_IMB_MAJOR, IMB_DRV_NAME, &IMB_fops);
	DEBUG("init_module: register_chrdev returned imb_major %d\n",imb_major);

	//
    //Since DEFAULT_IMB_MAJOR = 0, ie let the kernel assign a major number,
	// the return value must be > 0.
	//If we had given a specific major number, then the return 
	//should be 0 to indicate success.
	//
    if (imb_major <= 0) { 
		printk(KERN_ERR "init_module: Unable to register_chrdev\n");
		return (-EBUSY);
	}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14))
#ifdef CONFIG_COMPAT
    {
       err = register_ioctl32_conversion(IOCTL_IMB_SEND_MESSAGE, IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_GET_ASYNC_MSG, IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_MAP_MEMORY, IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_UNMAP_MEMORY, IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_SHUTDOWN_CODE, IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_REGISTER_ASYNC_OBJ, 
					IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_DEREGISTER_ASYNC_OBJ,
					IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_CHECK_EVENT, IMB_ioctl32);
       err |= register_ioctl32_conversion(IOCTL_IMB_POLL_ASYNC, IMB_ioctl32);
       printk("register_ioctl32_conversion ret = %d\n",err);
    }
#endif
#endif

	//
	// We don't allocate the space for the deviceExtension structure here.
	// It is now done in open. MR 11/30/00.
	//
	
	//
	// Initialize Wait Queue
	//
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  &kthreadStruct.queue = NULL;
#else
  init_waitqueue_head(&kthreadStruct.queue);
#endif

    kcs_command_register = kcsbase + spacing;
    kcs_status_register = kcsbase + spacing;
    kcs_datain_register = kcsbase;
    kcs_dataout_register = kcsbase;
    _smbaddr = smbaddr << 1;  /* use smbaddr * 2 */
    DEBUG("init_module: kcsbase=%x smbaddr=%x spacing=%d\n",
		kcsbase,smbaddr,spacing);	

    if (fsmbus && ftimestampthread) {  /* SMBus, & create extra thread */
	//
        // (if smbus)
	// Create Kernel Thread which updates mBMC timestamp
	//
	init_MUTEX_LOCKED(&kthreadStruct.semap);
	kthreadStruct.terminate = 0;
	OS_kernel_thread((int(*)(void *))UpdateTimeStampThread,
					 (void *)(&kthreadStruct) , 
					 CLONE_FS | CLONE_FILES | CLONE_SIGHAND);
	down(&kthreadStruct.semap);
	
	if(kthreadStruct.threadFail){
		DEBUG("init_module: init_module failed\n");		
		return (-ECHILD);
	}
    } 
	
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,16))
    err = atomic_notifier_chain_register(&panic_notifier_list,&imb_panic_block);
#else
    err = notifier_chain_register(&panic_notifier_list, &imb_panic_block);
#endif
    if (debug) printk("imb: notifier_chain_register rv = %d\n",err);

    DEBUG("init_module: init_module successful\n");	
    return (SUCCESS);
    
} //init_module

//////////////////////////////////////////////////////////
//  cleanup_module
//////////////////////////////////////////////////////////
/*F*
//  Name:       cleanup_module
//  Purpose:    supports unloading of the driver
//  Context:    called to unload the driver - remmod command
//  Returns:    zero on success, negative number on failure
//  Parameters: none
//  Notes:      none
*F*/
void
cleanup_module(void)
{

	//cleanup_module no longer frees up the deviceExtension.
	//It was already done in the close processing. MR 11/30/00.
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,16))
    atomic_notifier_chain_unregister(&panic_notifier_list, &imb_panic_block); 
#else
    notifier_chain_unregister(&panic_notifier_list, &imb_panic_block); 
#endif

    if (fsmbus && ftimestampthread) {  /* SMBus, & create extra thread */
	// Politely ask the update SDR Timestamp to die:
	init_MUTEX_LOCKED(&kthreadStruct.semap);
	kthreadStruct.terminate = 1;
	OS_wake_up_interruptible(&kthreadStruct.queue);		 
	down(&kthreadStruct.semap);
    }

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14))
#ifdef CONFIG_COMPAT
    {
       int err = 0;
       err = unregister_ioctl32_conversion(IOCTL_IMB_SEND_MESSAGE);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_GET_ASYNC_MSG);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_MAP_MEMORY);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_UNMAP_MEMORY);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_SHUTDOWN_CODE);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_REGISTER_ASYNC_OBJ);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_DEREGISTER_ASYNC_OBJ);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_CHECK_EVENT);
       err |= unregister_ioctl32_conversion(IOCTL_IMB_POLL_ASYNC);
    }
#endif
#endif
	
	//Tell the kernel to dis-engage us. We are going bye-bye.
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,18))
    unregister_chrdev(imb_major, IMB_DRV_NAME);   /*returns void*/
#else
    if (unregister_chrdev(imb_major, IMB_DRV_NAME) < 0)
	printk(KERN_ERR "IMB: unable to unregister_chrdev\n");
#endif
	
    DEBUG("imb: cleanup_module says bye-bye\n");
} //cleanup_module

/////////////////////////////////////////////////////////////////
// DriverEntry
/////////////////////////////////////////////////////////////////
/*F*
//  Name:       DriverEntry
//  Purpose:    Initialize the deviceExtension state structure
//  Context:    Called by init_module routine
//  Returns:    0 on success and a negative error number on failure.
//  Parameters:
//              deviceExtension -- state structure for driver
//  Notes:      none
*F*/

#define IPROBE_START   0xff

static int
DriverEntry( PDEVICE_EXTENSION deviceExtension)
{
    ImbInterface 			**imbIf;
    extern ImbInterface 	*ImbInterfaces[];
    static int iProbe = IPROBE_START;
    int i;
    char *pstr;
    int addr;

    DEBUG("IMB DriverEntry. deviceExtension = %p \n", deviceExtension);

    // Check to see if we have an interface that we recognize
    //
    if (iProbe == IPROBE_START) {
      i = 0;
      for( imbIf = ImbInterfaces ; *imbIf != NULL ; imbIf++ ) {
        if( (*imbIf)->ifProbe() == TRUE ) {
            DEBUG("IMB DriverEntry: Probe %d success\n",i);
            /* i==0 means KCS, i==1 means SMIC, i==2 means SMBus */
            switch (i) {
               case 2: fsmbus = 1;  /* probed SMBus ok */
                   pstr = "SMBus";
                   addr = _smbaddr;
                   break;
               case 1: fsmbus = 0;  /* probed SMIC ok */
                   pstr = "SMIC";
                   addr = smicaddr;
                   break;
               case 0: fsmbus = 0;  /* probed KCS ok */
               default:
                   pstr = "KCS";
                   addr = kcsbase;
                   break;
            }
            printk(KERN_INFO "IMB driver v%s found BMC via %s at 0x%x\n",imb_ver,pstr,addr);
            break;
	}
        i++;
      }
      if( *imbIf == NULL ) {
        printk(KERN_ERR "IMB DriverEntry: ifProbe failed\n");
	return(FAILURE);
      }
      iProbe = i;
    } else {  /* use previously discovered interface */
      DEBUG("IMB DriverEntry: iProbe=%d\n",iProbe);
      imbIf = ImbInterfaces;
      imbIf += iProbe;
    }
    g_sendBmcRequest  =  (*imbIf)->sendBmcRequest;

    //
    // Set up the device extension.
    //
    deviceExtension->unloadingDriver    = 0;

    //Set up the mesgTimer and the TimerDpcFuncM (similar to KeInitializeTimer
    //and KeInitializeDpc for NT and unlike timeout in Unix). 
    // The expires field will get set when starting timer

    DEBUG("IMB DriverEntry. init_timer for mesgTimer\n");

    init_timer( &deviceExtension->mesgTimer );
    deviceExtension->mesgTimer.data = (unsigned long)deviceExtension;
    deviceExtension->mesgTimer.function = TimerDpcFuncM;

    deviceExtension->timeOutFlagM	= 0;
    deviceExtension->shutdownCode       = SD_NO_ACTION;
    deviceExtension->imbSeqNo           = 0;
    deviceExtension->imbIf              = *imbIf;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  deviceExtension->asyncQSema         = NULL;
#else
  init_waitqueue_head(&deviceExtension->asyncQSema);
#endif

    //
    // initialize the request semaphore
    //
    init_MUTEX(&deviceExtension->imbreq_mutex);

	IMB_LOCK_ALLOC(deviceExtension->requestSemaphor);
	if (deviceExtension->requestSemaphor == NULL) {
		printk(KERN_ERR "imb: DriverEntry - req - IMB_LOCK_ALLOC failed\n");
		return(FAILURE);
	}
    //
    // initialize the transaction semaphore
    //
	IMB_LOCK_ALLOC(deviceExtension->transactionSema);
	if (deviceExtension->transactionSema == NULL) {
		printk(KERN_ERR "imb: DriverEntry - trans - IMB_LOCK_ALLOC failed\n");
		return(FAILURE);
	}

    //
    // Allocate space and initialize the async messaging subsystem
	// Note: kmalloc is gauranteed to succeed as we are using GFP_KERNEL
	// which will let it sleep if necessary to get us the memory.
    //
    deviceExtension->asyncQ.queue = (ImbAsyncEntry *) kmalloc(
				IMB_ASYNC_QUEUE_SIZE * sizeof(ImbAsyncEntry),
						 GFP_KERNEL);
    deviceExtension->asyncQ.queueSize = IMB_ASYNC_QUEUE_SIZE;


    ImbAsyncInit( &deviceExtension->asyncQ );
    
    // assume by default old IPMI version and later detect the actual one.
    deviceExtension->bmcVer =   IPMI_OLD_VER;
    getBmcIpmiVersion(deviceExtension);

    DEBUG("IMB DriverEntry: Allocations etc went well\n");
    return(SUCCESS);
} //DriverEntry

//////////////////////////////////////////////////////////////////
// DriverExit
//////////////////////////////////////////////////////////////////
/*F*
//  Name:       DriverExit
//  Purpose:    OS-independent driver detach/unload routine
//  Context:    called by IMBdetach (IMB_release)
//  Returns:    void
//  Parameters: deviceExtension -- driver state structure
//  Notes:      none
*F*/
static void
DriverExit(PDEVICE_EXTENSION deviceExtension)
{
    DEBUG("imb DriverExit. called to free deviceExtension fields\n");

	//Cancel our mesgTimer
    if (timer_pending( &deviceExtension->mesgTimer)) {
	DEBUG("imb DriverExit. mesgTimer timer_pending true so delete\n");
	del_timer( &deviceExtension->mesgTimer);
    }
	
    //
    //De-allocate all the space we allocated in the DriverEntry
	IMB_LOCK_DEALLOC(deviceExtension->requestSemaphor);

	//
	//First clean up the asyncQ's fields and then the queue itself
	// IMB_LOCK_DEALLOC(deviceExtension->asyncQ.syncSem);
	IMB_LOCK_DEALLOC(deviceExtension->asyncQ.waitEvent.plock);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
	//Incase we get here and a thread from user-space is still blocked
	if (deviceExtension->asyncQ.waitEvent.waitOn) {
		OS_wake_up_interruptible(&deviceExtension->asyncQ.waitEvent.waitOn);
		deviceExtension->asyncQ.waitEvent.waitOn = NULL;
	}
#else
	OS_wake_up_interruptible(&deviceExtension->asyncQ.waitEvent.waitOn);
#endif	

	if (timer_pending( &deviceExtension->asyncQ.waitEvent.asyncMesgTimer)) {
		DEBUG("imb DriverExit. asyncMesgTimer timer_pending true so delete\n");
		del_timer(&deviceExtension->asyncQ.waitEvent.asyncMesgTimer);
	} 
	kfree(deviceExtension->asyncQ.queue);
	
    return;
} //DriverExit


//////////////////////////////////////////////////////////////////
//  IMB_open
//////////////////////////////////////////////////////////////////
/*F*
//  Name:       IMB_open
//  Purpose:    open the device for use by user
//  Context:    called when user calls open() on device node
//  Returns:    0 on success, an appropriate errno on failure
//  Parameters: device number, flags, open type, credential pointer
//  Notes:      none
*F*/
/*ARGSUSED*/
static int
IMB_open(struct inode *inodep, struct file *filep)
{
	PDEVICE_EXTENSION   deviceExtension;
	int imb_minor;

	imb_minor = MINOR(inodep->i_rdev);

	// For now we assume that only one minor exists for our driver
	// and we want to validate that assumption. In the future if
	// we support multiple devices, we would make use of this number
	// to associate each open with the proper device extension struct.

	if (imb_minor != 0) {
		printk(KERN_ERR "IMB_open: open failed due to invalid minor device ID\n");
		return(-ENODEV);	//ENODEV: No such device
	}

    //
    // Associate the private_data pointer for this open to the DeviceExtension
	// being allocated. All future access to the driver will use the 
	// private_data pointer. Mahendra 11/30/00.
    //
    deviceExtension = (PDEVICE_EXTENSION)kmalloc( sizeof(DEVICE_EXTENSION), 
							GFP_KERNEL);
	memset(deviceExtension, sizeof(DEVICE_EXTENSION), 0);

	DEBUG("IMB_open allocated deviceExtension addr %p\n", deviceExtension);

    if (DriverEntry(deviceExtension) == FAILURE) {
	printk(KERN_ERR "IMB_open: DriverEntry failure. unregister device\n");
	kfree(deviceExtension);
	unregister_chrdev(imb_major, IMB_DRV_NAME);
	return(-EIO);
    }
    filep->private_data = deviceExtension ;

    if (filep->private_data == NULL) {
		printk(KERN_ERR "IMB_open: open failed due to invalid device struture\n");
        return(-ENXIO);		//ENXIO: No such device or address
	}

    // g_deviceExtension = deviceExtension;
	DEBUG("IMB_open: Minor %d, Device %p\n",imb_minor,filep->private_data);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
	MOD_INC_USE_COUNT;
#else
        /* Linux 2.6 kernel module logic */
        use_count++;
#endif
    return(SUCCESS);
} //IMBopen


///////////////////////////////////////////////////////////////////////
// IMB_release
///////////////////////////////////////////////////////////////////////
/*F*
//  Name:       IMB_release  (was IMB_close)
//  Purpose:    closes opened device node
//  Context:    called when user calls close() on open device node
//  Returns:    0 on success, appropriate errno on failure
//  Parameters: device number, flags, type, credential pointer
//  Notes:      none
*F*/
/*ARGSUSED*/
static int
IMB_release(struct inode *inodep, struct file *filep)
{

    //
    // Check the pointer to the device extension.
    //

    if (filep->private_data == NULL)
        return(-ENXIO);		//ENXIO: No such device or address

    DEBUG("IMB_release: DeviceExtension %p\n", filep->private_data);

    // De-allocate the DPC syncronization variable so that it can
    // be allocated the next time DPC thread starts up. Otherwise
    // stoppping and starting basebrd will not work. MR
    ImbUnixAsyncDeallocate(filep->private_data);

	//Lets free up the data structure.
    DriverExit(filep->private_data);
	kfree(filep->private_data);


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
	MOD_DEC_USE_COUNT; //If the module inuse count went to 0, driver will
					   //be unloaded.
#else
    use_count--;
    if (use_count > 0) {
        DEBUG("IMB_release: Device driver use_count=%d \n",use_count);
    }
#endif
    return(SUCCESS);
} //IMB_release


///////////////////////////////////////////////////////////////////////
// IMB_ioctl
///////////////////////////////////////////////////////////////////////
/*F*
//  Name:       IMB_ioctl
//  Purpose:    user interface to driver
//  Context:    called when user calls ioctl() on open device node
//  Returns:    0 on success, errno on failure, error number on
//              recoverable error
//  Parameters: Compiles with the kernel driver entry prototype.
//  Notes:		No locks held since only one ioctl can be outstanding at a time
//				Multi-threaded apps like cif are sync'ed in libimbapi.
//			
//             
*F*/

/*ARGSUSED*/
static int
IMB_ioctl(struct inode *inodep, struct file *filep, unsigned int cmd, 
			unsigned long arg)
{
    PDEVICE_EXTENSION deviceExtensionP = filep->private_data;
    IMB_STATUS status = STATUS_SUCCESS;		
    DWORD inputBufferLength, outputBufferLength, length;	
    struct smi  smi; // ,*smi_p;
    int szsmi;
   
    DEBUG("IMB_ioctl: cmd 0x%x arg 0x%lx\n", cmd, arg);

    if (deviceExtensionP == NULL)
        return(-ENXIO);			//ENXIO: No such device or address
    if ((caddr_t)arg == NULL)
        return(-EINVAL);		//EINVAL: Invalid argument
    szsmi = sizeof(smi);

#ifdef DEBUG_EXTRA 
    {
       int i;
       /* detect if this command is in 32-bit mode */
       #define NCMDS  9
       int cmds[NCMDS] = { IOCTL_IMB_SEND_MESSAGE, IOCTL_IMB_GET_ASYNC_MSG,
           IOCTL_IMB_MAP_MEMORY, IOCTL_IMB_UNMAP_MEMORY,
           IOCTL_IMB_SHUTDOWN_CODE, IOCTL_IMB_CHECK_EVENT,
           IOCTL_IMB_REGISTER_ASYNC_OBJ, IOCTL_IMB_DEREGISTER_ASYNC_OBJ,
           IOCTL_IMB_POLL_ASYNC };
       for (i = 0; i < NCMDS; i++) if (cmds[i] == cmd) break;
       if (i < NCMDS) {  /* 32-bit compatibility mode */
           /* sizeof(struct smi) =40 in i386, =80 in x86_64 */
           printk("cmd i = %d\n",i);
           // szsmi = 40;
           // return(-EINVAL);  /* pointers will be wrong */
       }
    }
#endif
    // First copy the smi data from user to kernel space
    if (copy_from_user((caddr_t)&smi, (caddr_t)arg, szsmi) == -1) {
		printk(KERN_ERR "imb_ioctl: copy_from_user failed\n");
        return(-EFAULT);
	}

    inputBufferLength  = smi.cbInBuffer;
    outputBufferLength = smi.cbOutBuffer; 
    DEBUG("IMB_ioctl: inputBufferLength = %ld outputBufferLength = %ld\n",
			inputBufferLength, outputBufferLength);
    // Note: Need to have special handling for 32-bit requests if 
    // CONFIG_COMPAT on 64-bit.
#ifdef DEBUG_EXTRA
     { 
        int j, n;
        char *tag;
        unsigned int *bi;
        unsigned char *b;

#ifndef LINUX_DEBUG
        printk("IMB_ioctl: inputBufferLength = %ld outputBufferLength = %ld\n",
			inputBufferLength, outputBufferLength);
#endif
        /* show arg/smi structure */
        bi = (unsigned int *)&smi;
        n = ( sizeof(struct smi) / sizeof(int) );
        printk("smi(%p), (sz=%d/szint=%d)->%d:\n", bi, sizeof(struct smi),
				sizeof(int),n);
        for (j = 0; j < n; j++) {
            switch(j) {
                case 0:   tag = "version  "; break;
                case 2:   tag = "reserved1"; break;
                case 4:   tag = "reserved2"; break;
                case 6:   tag = "ntstatus "; break;
                case 8:   tag = "pinbuf "; break;
                case 10:  tag = "sinbuf "; break;
                case 12:  tag = "poutbuf"; break;
                case 14:  tag = "soutbuf"; break;
                case 16:  tag = "cbRet  "; break;
                case 18:  tag = "pOverlap"; break;
                default:  tag = "other  "; 
            }
            printk("bi[%d]%s: %08x \n",j,tag,bi[j]);
        }
      
        /* show inbuf pointer/size */
        printk("inbuf(%p), (sz=%d):\n", smi.lpvInBuffer, inputBufferLength);
        /* data not yet copied from user */
        // for (j = 0; j < n; j++) {
        //     printk("b[%d]: %02x \n",j,b[j]);
        // }
     }
#endif

    //
    // Case on the ioctl sent down by the user
    //
    switch ( cmd ) {
    
		case IOCTL_IMB_POLL_ASYNC:
			// sms buffer message processing
			ImbAsyncMessageProcess(deviceExtensionP); 
			break;

		case IOCTL_IMB_GET_ASYNC_MSG:
		//
		// check parameters
		//
		DEBUG("IMB_ioctl: IOCTL_IMB_GET_ASYNC_MSG\n");
		if( inputBufferLength < sizeof( ImbAsyncRequest ) ) {
			// 
			// Too short an input buffer than we expected - invalid error
			//
			printk("IMB_ioctl: IOCTL_IMB_GET_ASYNC_MSG - InBufLen too short\n");
			return(-EINVAL);
		} else if ( outputBufferLength < MIN_ASYNC_RESP_SIZE )   {
			// 
			// Too short an output buffer than we expected - invalid error
			//
			printk("IMB_ioctl: IOCTL_IMB_GET_ASYNC_MSG - OutBufLen too short\n");
			return(-EINVAL);
		} else {
			//
			// get his request and try to get a queued message, If none are 
			// available and the timeout is non-zero, the request will sleep 
			// until timeout or message available
			//
			
			unsigned char AsyncReq[MAX_ASYNC_RESP_SIZE];
			unsigned char AsyncResp[MAX_ASYNC_RESP_SIZE];
			ImbAsyncRequest  *pAsyncReq;
			ImbAsyncResponse *pAsyncResp;

			pAsyncReq = (ImbAsyncRequest *)AsyncReq;
			pAsyncResp = (ImbAsyncResponse *)AsyncResp;

			//
			// Now we copy the Input Buffer data itself from user space.
			//
			if (copy_from_user( (void *)AsyncReq, (void *)smi.lpvInBuffer, 
						sizeof(ImbAsyncRequest)) == -1) {
				printk(KERN_ERR "IMB_ioctl: copy_from_user of lpvInBuffer failed\n");
				return(-EFAULT);
			}

			length = outputBufferLength;

			status = ImbAsyncGetMessage( pAsyncReq, pAsyncResp,&length,
										&deviceExtensionP->asyncQ );

			//
			// Now copy response and length back into user-space
			//
			
			if (copy_to_user( 	(void *)smi.lpvOutBuffer,	
								(void *)pAsyncResp,  
								length
							) == -1) {
				printk(KERN_ERR "IMB_ioctl: IOCTL_IMB_GET_ASYNC_MSG \
									copy_to_user to lpvOutBuffer failed\n");
				return(-EFAULT);
			}

			if ( copy_to_user(	(void *)smi.lpcbBytesReturned,
							  	(void *)&length,  
								sizeof(smi.lpcbBytesReturned)
							  ) == -1)  {
				printk(KERN_ERR "IMB_ioctl: IOCTL_IMB_GET_ASYNC_MSG \
									 copy_to_user to lpvOutBuffer failed\n");
				return(-EFAULT);
			}
        }
        break;

    case IOCTL_IMB_SEND_MESSAGE:
	DEBUG("IMB_ioctl: IOCTL_IMB_SEND_MESSAGE(inlen=%d,outlen=%d)\n",
		inputBufferLength,outputBufferLength);
        //
        // check parameters
        //
        if ( (inputBufferLength < MIN_IMB_REQ_BUF_SIZE) || 
        	 (outputBufferLength < MIN_IMB_RESP_BUF_SIZE) )   {
            //
            // Input or Output buffer is too small
            //
			printk("IMB_ioctl: IOCTL_IMB_SEND_MESSAGE - Buffer Len too short\n");
            return(-EINVAL);
        } else {
           //
           // let the command processing check the request contents
           //
		// probably bigger than needed check later 
		unsigned char imbReqBuffer[MAX_IMB_RESPONSE_SIZE + 8];
		unsigned char imbRespBuffer[MAX_IMB_RESPONSE_SIZE + 8];

    		ImbRequestBuffer   *pImbReq;
    		ImbResponseBuffer  *pImbResp;

		pImbReq = (ImbRequestBuffer *)imbReqBuffer;
		pImbResp = (ImbResponseBuffer *)imbRespBuffer;

		//
		// Now we copy the Input Buffer data itself from user space.
		//
		if (copy_from_user( (void *)pImbReq, (void *)smi.lpvInBuffer,  
						inputBufferLength) == -1) {
			printk(KERN_ERR "IMB_ioctl: copy_from_user ImbRequestBuffer failed\n");
			return(-EFAULT);
		}

#ifdef DEBUG_EXTRA
		{ 
		   int j;
	           printk("IMB_ioctl: IOCTL_IMB_SEND_MESSAGE buffer:\n");
        	   for (j = 0; j < inputBufferLength; j++) 
        	      printk("b[%d]: %02x \n",j,imbReqBuffer[j]);
                }
#endif
		//
    		// Setup the timer for this request and get the 
    		// request semaphore
    		ImbOsStartReq( pImbReq->timeOut, deviceExtensionP );

    		if( (DWORD)(pImbReq->req.dataLength + MIN_IMB_REQ_BUF_SIZE) > 
							inputBufferLength ) {
				//
				//cleanup and return
				//
	    		ImbOsEndReq( deviceExtensionP );
			printk("IMB_ioctl: IOCTL_IMB_SEND_MESSAGE failed\n");
				return(-EINVAL);
		} else {
			length = outputBufferLength - MIN_IMB_RESP_BUF_SIZE;


				//
				// do the work of sending the request
				//
			status = SendImbRequest(pImbReq, 
						pImbResp, 
						&length,
						deviceExtensionP);

			/* status E0040007 = IMB_IF_RECEIVE_TIMEOUT */
			if( status == IMB_SUCCESS ) {
					//
					//Now copy back the imb response and length to user space
					//
				if (copy_to_user((void *)smi.lpvOutBuffer,
						 (void *)pImbResp,  
						 length
						) == -1) {
					printk(KERN_ERR "IMB_ioctl: copy_to_user to lpvOutBuffer failed\n");
					return(-EFAULT);
				}

				if ( copy_to_user((void *)smi.lpcbBytesReturned,
						  (void *)&length,  
						  sizeof(smi.lpcbBytesReturned) 
						  ) == -1 ) {
					printk(KERN_ERR "IMB_ioctl: copy_to_user to lpcbBytesReturned failed\n");
					return(-EFAULT);
				}
			} //IMB_SUCCESS

				//
				// All went well. Now clean up
				//
			DEBUG("IMB_ioctl: IOCTL_IMB_SEND_MESSAGE done\n");
    			ImbOsEndReq( deviceExtensionP );
			} //else
	    }
        break;

    case IOCTL_IMB_SHUTDOWN_CODE:
        {
			ShutdownCmdBuffer shutdownCmd;

			if (copy_from_user( (void *)&shutdownCmd, (void *)smi.lpvInBuffer, 
						sizeof(ShutdownCmdBuffer)) == -1) {
				printk(KERN_ERR "IMB_ioctl: copy_from_user of shutdownCmd failed\n");
				return(-EFAULT);
			}

            //
            // check input parameters
            //
            if ( inputBufferLength < sizeof(ShutdownCmdBuffer) ) {
				printk(KERN_ERR "IMB_ioctl: IOCTL_IMB_SHUTDOWN_CODE InBufLen short\n");
                return(-EINVAL);
            }
            //
            // guarentee valid code
            //
            if (    shutdownCmd.code != SD_POWER_OFF   &&
                    shutdownCmd.code != SD_RESET       &&
                    shutdownCmd.code != SD_NO_ACTION       ) {

                deviceExtensionP->shutdownCode = SD_NO_ACTION;
                status = (long)INVALID_INPUT_BUFFER;
                break;
            }

            deviceExtensionP->shutdownCode = shutdownCmd.code;
            deviceExtensionP->delayTime    = shutdownCmd.delayTime;

			ImbShutdown(deviceExtensionP); // never return from here right?
            status = STATUS_SUCCESS;
        }

        break;
		
		//For DPC thread to register for anync messages.

		case IOCTL_IMB_REGISTER_ASYNC_OBJ:
			if (ImbUnixAsyncInit (deviceExtensionP) != 0) {
				int dummy;

				status = STATUS_SUCCESS;
				//
				//Now copy back the  a bogus imb response and length to 
				//user space since the imbapi expects it. We can use this
				//later to return a more meaningful value if necesssary.
				//
				dummy = 0xbabe;
				length = sizeof(dummy); /*ARCress 11/29/05*/
				if (copy_to_user((void *)smi.lpvOutBuffer,
						 (void *)&dummy,  
							length
							) == -1) {
					printk(KERN_ERR "IMB_ioctl: copy_to_user to lpvOutBuffer failed\n");
					return(-EFAULT);
				}
				
				dummy = sizeof(int); //sending back an int above.
				if ( copy_to_user((void *)smi.lpcbBytesReturned,
						  (void *)&dummy,  
							sizeof(dummy)
							  ) == -1 ) {
					printk(KERN_ERR "IMB_ioctl: copy_to_user to lpcbBytesReturned failed\n");
					return(-EFAULT);
				}
			}
			else
				status = STATUS_UNSUCCESSFUL; //MR is this a good status?
		break;

		case IOCTL_IMB_DEREGISTER_ASYNC_OBJ:
			ImbUnixAsyncDeallocate(deviceExtensionP);
			status = STATUS_SUCCESS;
		break;

		case IOCTL_IMB_CHECK_EVENT:
			//app thread gets blocked for async events here till an
			// event is received and unblocked by ImbAsyncQueueMessage
			ImbUnixAsyncLock(deviceExtensionP);
		break;

		//This ioctl is commented out in imbapi.c 
		//case IOCTL_IMB_GET_DPC_MSG:

		// mmap does this for us, we don't use an ioctl for these. MR
		//    case IOCTL_IMB_MAP_MEMORY : 
		//    case IOCTL_IMB_UNMAP_MEMORY: 


		default:
			printk(KERN_ERR "IMB_ioctl: ioctl cmd  0x%x not supported\n", cmd);
			return(-EINVAL);
    } // switch (cmd)

    return(status);
} //IMB_ioctl

#ifdef CONFIG_COMPAT
/*
 * int ioctl(   struct inode  *inode,
 *              struct file   *file,
 *              unsigned int  cmd,
 *              unsigned long arg); 
 * NOTE DIFFERENCES:
 * int ioctl32( unsigned int fd, 
 *              unsigned int cmd,
 *              unsigned long arg, 
 *              struct file *filep) 
 */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14))
static long 
IMB_ioctl32(struct file *filep, unsigned int cmd, 
			unsigned long arg )
#else
static int 
IMB_ioctl32( unsigned int fd, unsigned int cmd, 
	     unsigned long arg, struct file *filep) 
#endif
{
    PDEVICE_EXTENSION deviceExtensionP;
    IMB_STATUS status = STATUS_SUCCESS;		
    DWORD inputBufferLength, outputBufferLength, length;	
    struct smi32  smi; // ,*smi_p;
    int szsmi;
   
    DEBUG("IMB_ioctl32: cmd 0x%x arg 0x%lx\n", cmd, arg);
    deviceExtensionP = filep->private_data;

    if (deviceExtensionP == NULL)
        return(-ENXIO);			//ENXIO: No such device or address
    if ((caddr_t)arg == NULL)
        return(-EINVAL);		//EINVAL: Invalid argument
    szsmi = sizeof(struct smi32);

    // First copy the smi data from user to kernel space
    if (copy_from_user((caddr_t)&smi, compat_ptr(arg), szsmi) == -1) {
		printk(KERN_ERR "imb_ioctl32: copy_from_user failed\n");
        return(-EFAULT);
	}

    inputBufferLength  = smi.cbInBuffer;
    outputBufferLength = smi.cbOutBuffer; 
    DEBUG("IMB_ioctl32: inputBufferLength = %ld outputBufferLength = %ld\n",
			inputBufferLength, outputBufferLength);
    // Note: Need to have special handling for 32-bit requests if 
    // CONFIG_COMPAT on 64-bit.
#ifdef DEBUG_EXTRA
     { 
        int j, n;
        char *tag;
        unsigned int *bi;
        unsigned char *b;

#ifndef LINUX_DEBUG
        printk("IMB_ioctl32: inputBufferLength=%ld outputBufferLength=%ld\n",
			inputBufferLength, outputBufferLength);
#endif
        /* show arg/smi structure */
        bi = (unsigned int *)&smi;
        n = ( sizeof(struct smi) / sizeof(int) );
        printk("smi(%p), (sz=%d/szint=%d)->%d:\n", bi, sizeof(struct smi),
				sizeof(int),n);
        for (j = 0; j < n; j++) {
            switch(j) {
                case 0:   tag = "version  "; break;
                case 2:   tag = "reserved1"; break;
                case 4:   tag = "reserved2"; break;
                case 6:   tag = "ntstatus "; break;
                case 8:   tag = "pinbuf "; break;
                case 10:  tag = "sinbuf "; break;
                case 12:  tag = "poutbuf"; break;
                case 14:  tag = "soutbuf"; break;
                case 16:  tag = "cbRet  "; break;
                case 18:  tag = "pOverlap"; break;
                default:  tag = "other  "; 
            }
            printk("bi[%d]%s: %08x \n",j,tag,bi[j]);
        }
      
        /* show inbuf pointer/size */
        printk("inbuf(%p), (sz=%d):\n", compat_ptr(smi.lpvInBuffer), inputBufferLength);
        /* data not yet copied from user */
        // for (j = 0; j < n; j++) {
        //     printk("b[%d]: %02x \n",j,b[j]);
        // }
     }
#endif

    //
    // Case on the ioctl sent down by the user
    //
    switch ( cmd ) {
    
		case IOCTL_IMB_POLL_ASYNC:
			// sms buffer message processing
			ImbAsyncMessageProcess(deviceExtensionP); 
			break;

		case IOCTL_IMB_GET_ASYNC_MSG:
		//
		// check parameters
		//
		DEBUG("IMB_ioctl32: IOCTL_IMB_GET_ASYNC_MSG\n");
		if( inputBufferLength < sizeof( ImbAsyncRequest ) ) {
			// 
			// Too short an input buffer than we expected - invalid error
			//
			printk("IMB_ioctl32: IOCTL_IMB_GET_ASYNC_MSG - InBufLen too short\n");
			return(-EINVAL);
		} else if ( outputBufferLength < MIN_ASYNC_RESP_SIZE )   {
			// 
			// Too short an output buffer than we expected - invalid error
			//
			printk("IMB_ioctl32: IOCTL_IMB_GET_ASYNC_MSG - OutBufLen too short\n");
			return(-EINVAL);
		} else {
			//
			// get his request and try to get a queued message, If none are 
			// available and the timeout is non-zero, the request will sleep 
			// until timeout or message available
			//
			
			unsigned char AsyncReq[MAX_ASYNC_RESP_SIZE];
			unsigned char AsyncResp[MAX_ASYNC_RESP_SIZE];
			ImbAsyncRequest  *pAsyncReq;
			ImbAsyncResponse *pAsyncResp;

			pAsyncReq = (ImbAsyncRequest *)AsyncReq;
			pAsyncResp = (ImbAsyncResponse *)AsyncResp;

			//
			// Now we copy the Input Buffer data itself from user space.
			//
			if (copy_from_user( (void *)AsyncReq, compat_ptr(smi.lpvInBuffer), 
						sizeof(ImbAsyncRequest)) == -1) {
				printk(KERN_ERR "IMB_ioctl32: copy_from_user of lpvInBuffer failed\n");
				return(-EFAULT);
			}

			length = outputBufferLength;

			status = ImbAsyncGetMessage( pAsyncReq, pAsyncResp,&length,
										&deviceExtensionP->asyncQ );

			//
			// Now copy response and length back into user-space
			//
			
			if (copy_to_user( compat_ptr(smi.lpvOutBuffer),	
						(void *)pAsyncResp,  
						length
					) == -1) {
				printk(KERN_ERR 
				       "IMB_ioctl32: IOCTL_IMB_GET_ASYNC_MSG "
				       "copy_to_user to lpvOutBuffer failed\n");
				return(-EFAULT);
			}

			if ( copy_to_user( compat_ptr(smi.lpcbBytesReturned),
					  	(void *)&length,  
						sizeof(smi.lpcbBytesReturned)
					  ) == -1)  {
				printk(KERN_ERR "IMB_ioctl32: IOCTL_IMB_GET_ASYNC_MSG "
					"copy_to_user to lpvOutBuffer failed\n");
				return(-EFAULT);
			}
        }
        break;

    case IOCTL_IMB_SEND_MESSAGE:
	DEBUG("IMB_ioctl32: IOCTL_IMB_SEND_MESSAGE(inlen=%d,outlen=%d)\n",
		inputBufferLength,outputBufferLength);
        //
        // check parameters
        //
        if ( (inputBufferLength < MIN_IMB_REQ_BUF_SIZE) || 
        	 (outputBufferLength < MIN_IMB_RESP_BUF_SIZE) )   {
            //
            // Input or Output buffer is too small
            //
			printk("IMB_ioctl32: IOCTL_IMB_SEND_MESSAGE - Buffer Len too short\n");
            return(-EINVAL);
        } else {
           //
           // let the command processing check the request contents
           //
		// probably bigger than needed check later 
		unsigned char imbReqBuffer[MAX_IMB_RESPONSE_SIZE + 8];
		unsigned char imbRespBuffer[MAX_IMB_RESPONSE_SIZE + 8];

    		ImbRequestBuffer   *pImbReq;
    		ImbResponseBuffer  *pImbResp;

		pImbReq = (ImbRequestBuffer *)imbReqBuffer;
		pImbResp = (ImbResponseBuffer *)imbRespBuffer;

		//
		// Now we copy the Input Buffer data itself from user space.
		//
		if (copy_from_user( (void *)pImbReq, compat_ptr(smi.lpvInBuffer),  
						inputBufferLength) == -1) {
			printk(KERN_ERR "IMB_ioctl32: copy_from_user ImbRequestBuffer failed\n");
			return(-EFAULT);
		}

#ifdef DEBUG_EXTRA
		{ 
		   int j;
	           printk("IMB_ioctl32: IOCTL_IMB_SEND_MESSAGE buffer:\n");
        	   for (j = 0; j < inputBufferLength; j++) 
        	      printk("b[%d]: %02x \n",j,imbReqBuffer[j]);
                }
#endif
		//
    		// Setup the timer for this request and get the 
    		// request semaphore
    		ImbOsStartReq( pImbReq->timeOut, deviceExtensionP );

    		if( (DWORD)(pImbReq->req.dataLength + MIN_IMB_REQ_BUF_SIZE) > 
							inputBufferLength ) {
				//
				//cleanup and return
				//
	    		ImbOsEndReq( deviceExtensionP );
			printk("IMB_ioctl32: IOCTL_IMB_SEND_MESSAGE failed\n");
				return(-EINVAL);
		} else {
			length = outputBufferLength - MIN_IMB_RESP_BUF_SIZE;


				//
				// do the work of sending the request
				//
			status = SendImbRequest(pImbReq, 
						pImbResp, 
						&length,
						deviceExtensionP);

			/* status E0040007 = IMB_IF_RECEIVE_TIMEOUT */
			if( status == IMB_SUCCESS ) {
					//
					//Now copy back the imb response and length to user space
					//
				if (copy_to_user(compat_ptr(smi.lpvOutBuffer),
						 (void *)pImbResp,  
						 length
						) == -1) {
					printk(KERN_ERR "IMB_ioctl32: copy_to_user to lpvOutBuffer failed\n");
					return(-EFAULT);
				}

				if ( copy_to_user(compat_ptr(smi.lpcbBytesReturned),
						  (void *)&length,  
						  sizeof(smi.lpcbBytesReturned) 
						  ) == -1 ) {
					printk(KERN_ERR "IMB_ioctl32: copy_to_user to lpcbBytesReturned failed\n");
					return(-EFAULT);
				}
			} //IMB_SUCCESS

				//
				// All went well. Now clean up
				//
			DEBUG("IMB_ioctl32: IOCTL_IMB_SEND_MESSAGE done\n");
    			ImbOsEndReq( deviceExtensionP );
			} //else
	    }
        break;

    case IOCTL_IMB_SHUTDOWN_CODE:
        {
			ShutdownCmdBuffer shutdownCmd;

			if (copy_from_user( (void *)&shutdownCmd, compat_ptr(smi.lpvInBuffer), 
						sizeof(ShutdownCmdBuffer)) == -1) {
				printk(KERN_ERR "IMB_ioctl32: copy_from_user of shutdownCmd failed\n");
				return(-EFAULT);
			}

            //
            // check input parameters
            //
            if ( inputBufferLength < sizeof(ShutdownCmdBuffer) ) {
				printk(KERN_ERR "IMB_ioctl32: IOCTL_IMB_SHUTDOWN_CODE InBufLen short\n");
                return(-EINVAL);
            }
            //
            // guarentee valid code
            //
            if (    shutdownCmd.code != SD_POWER_OFF   &&
                    shutdownCmd.code != SD_RESET       &&
                    shutdownCmd.code != SD_NO_ACTION       ) {

                deviceExtensionP->shutdownCode = SD_NO_ACTION;
                status = (long)INVALID_INPUT_BUFFER;
                break;
            }

            deviceExtensionP->shutdownCode = shutdownCmd.code;
            deviceExtensionP->delayTime    = shutdownCmd.delayTime;

			ImbShutdown(deviceExtensionP); // never return from here right?
            status = STATUS_SUCCESS;
        }

        break;
		
		//For DPC thread to register for anync messages.

		case IOCTL_IMB_REGISTER_ASYNC_OBJ:
			if (ImbUnixAsyncInit (deviceExtensionP) != 0) {
				int dummy;

				status = STATUS_SUCCESS;
				//
				//Now copy back the  a bogus imb response and length to 
				//user space since the imbapi expects it. We can use this
				//later to return a more meaningful value if necesssary.
				//
				dummy = 0xbabe;
				length = sizeof(dummy); /*ARCress 11/29/05*/
				if (copy_to_user(compat_ptr(smi.lpvOutBuffer),
						 (void *)&dummy,  
							length
							) == -1) {
					printk(KERN_ERR "IMB_ioctl32: copy_to_user to lpvOutBuffer failed\n");
					return(-EFAULT);
				}
				
				dummy = sizeof(int); //sending back an int above.
				if ( copy_to_user(compat_ptr(smi.lpcbBytesReturned),
						  (void *)&dummy,  
							sizeof(dummy)
							  ) == -1 ) {
					printk(KERN_ERR "IMB_ioctl32: copy_to_user to lpcbBytesReturned failed\n");
					return(-EFAULT);
				}
			}
			else
				status = STATUS_UNSUCCESSFUL; //MR is this a good status?
		break;

		case IOCTL_IMB_DEREGISTER_ASYNC_OBJ:
			ImbUnixAsyncDeallocate(deviceExtensionP);
			status = STATUS_SUCCESS;
		break;

		case IOCTL_IMB_CHECK_EVENT:
			//app thread gets blocked for async events here till an
			// event is received and unblocked by ImbAsyncQueueMessage
			ImbUnixAsyncLock(deviceExtensionP);
		break;

		//This ioctl is commented out in imbapi.c 
		//case IOCTL_IMB_GET_DPC_MSG:

		// mmap does this for us, we don't use an ioctl for these. MR
		//    case IOCTL_IMB_MAP_MEMORY : 
		//    case IOCTL_IMB_UNMAP_MEMORY: 


		default:
			printk(KERN_ERR "IMB_ioctl32: ioctl cmd  0x%x not supported\n", cmd);
			return(-EINVAL);
    } // switch (cmd)

    return(status);
} //IMB_ioctl32
#endif


/////////////////////////////////////////////////////////////
//  IMBmmap
/////////////////////////////////////////////////////////////
/*F*
//  Name:       IMBmmap
//  Purpose:    maps physical memory for user
//  Context:    called when user calls mmap() on device
//  Returns:    
//  Parameters: device number, offset (from phys addr 0, not from beginning
//              of device!), and protection info
//  Notes:      none
*F*/
static int
IMBmmap(struct file *filep, struct vm_area_struct *vma)
{
    int   ret = 0;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
    off_t    offset = vma->vm_offset;
#else	
    // off_t    offset = (vma->vm_pgoff << PAGE_SHIFT);
    unsigned long offset = (vma->vm_pgoff << PAGE_SHIFT);
#endif

    DEBUG("IMBmmap:vm_start 0x%lx vm_end 0x%lx\n",vma->vm_start,vma->vm_end);

    if (offset < 0) {
        printk(KERN_ERR "IMBmmap: called with invalid offset\n");
        return(-1);
    }

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10))
    if (remap_pfn_range(vma, vma->vm_start, offset,
                         vma->vm_end - vma->vm_start, vma->vm_page_prot))
#elif ((LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,3)) || defined(LINUXAS))
    if (remap_page_range(vma, vma->vm_start, offset,
                         vma->vm_end - vma->vm_start, vma->vm_page_prot))
#else
    if (remap_page_range(vma->vm_start, offset,
                         vma->vm_end - vma->vm_start, vma->vm_page_prot))
#endif
	{
		printk(KERN_ERR "IMBmmap: mapping failed\n");
		return(-EAGAIN);
	}
	
	DEBUG("IMBmmap: mmap success\n");

	//vma->vm_inode = what_goes_here;

    return(ret);
} //IMBmmap


/////////////////////////////////////////////////////////////////////
// ImbShutdown
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbShutdown
//  Purpose:    This is the dispatch routine for processing shutdown request.
//  Context:    Called by operating system during system exit.
//  Returns:    STATUS_SUCCESS
//  Parameters:
//              deviceExtension    Pointer to driver object created by system.
//  Notes:      none
*F*/
static NTSTATUS
ImbShutdown ( PDEVICE_EXTENSION deviceExtension)
{
    NTSTATUS            status;

    switch ( deviceExtension->shutdownCode ) {

    case SD_POWER_OFF:
    case SD_RESET   :
        //
        //
        // Get the request semaphore, to  avoid colliding with the
        // main request thread. Also starts the request timer
        //
        ImbOsStartReq( DEFAULT_MESSAGE_TIMEOUT, deviceExtension );
        //
        // execute the action
        //
        ImbShutdownSystem(  deviceExtension->shutdownCode,
                            deviceExtension->delayTime,
                            deviceExtension
                         );
        //
        // release the request semaphore
        //
        ImbOsEndReq( deviceExtension );

        status = STATUS_SUCCESS;
        break;

    case SD_NO_ACTION:

        status = STATUS_SUCCESS;
        break;

    default:
        status = INVALID_INPUT_BUFFER ;
        break;
    }
    return status;
} //ImbShutdown

/////////////////////////////////////////////////////////////////////
// Delay
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       Delay
//  Purpose:    Busy Loop
//  Context:   
//  Returns:   
//  Parameters:
//  Notes:     
*F*/
VOID Delay(
		   DWORD delayTime //in usec 10(-6)
		   )
{
	// If delayTime is more than 1000 usecs we should use mdelay which
	// is for micro-sec delays and calls udelay multiple times.
	if (delayTime <= 1000)
		udelay((ULONG)(delayTime));
	else
		mdelay((ULONG)(delayTime/1000)); //close enough approximation
}

/////////////////////////////////////////////////////////////////////
// UpdateTimeStampThread
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       UpdateTimeStampThread
//  Purpose:    Thread which sleeps a lot !
//				It wakes up once every 1/2 an hour to update 
//				the mBMC timestamp.
//  Context:   
//  Returns:   
//  Parameters:
//  Notes:     
*F*/
void UpdateTimeStampThread(void * arg)
{
		kthread_t * pkthread = (kthread_t *) arg;
		unsigned long CurrentTime;
	int timedout;
//		DWORD	currentTime;
//		int ret;
		
		// Since the device extension is allocated again and again
		// for each DriverEntry, we need to reproduce it here:
		PDEVICE_EXTENSION   deviceExtension;
		
		up(&pkthread->semap);
        
        deviceExtension = (PDEVICE_EXTENSION)kmalloc(sizeof(DEVICE_EXTENSION), 
							GFP_KERNEL);
	memset(deviceExtension, sizeof(DEVICE_EXTENSION), 0);

        DEBUG("UpdateTimeStampThread: deviceExtension %p\n", deviceExtension);
	deviceExtension->unloadingDriver    = 0;

	pkthread->threadFail = 0;

	if (DriverEntry(deviceExtension) == FAILURE) {
			printk(KERN_ERR "IMB_open: DriverEntry failure. unregister device\n");
			kfree(deviceExtension);
			unregister_chrdev(imb_major, IMB_DRV_NAME);
			pkthread->threadFail = 1;
			return;
		}
		
       // Loop is done untill Thread is Terminated
	while (!pkthread->terminate) 
        {          
			// The real work 
            DEBUG("UpdateTimeStampThread: Thread is up\n");     
            // Read the RT Clock
	    CurrentTime = GetRtcTime();
	    // Update Timestamp			
	    ImbUpdateSDRTime(&CurrentTime, deviceExtension);
	    // Fall asleep for 1/2 an hour
	    timedout = OS_interruptible_sleep_on_timeout(&pkthread->queue, UPDATE_TIMESTAMP);
            if (timedout == 0) {
                /* interrupted, should be exiting */
                DEBUG("UpdateTimeStampThread: interrupted\n");     
                break;
            } else { /* timed out normally */;  }
        };
        
        // Free the Device Extension
        DriverExit(deviceExtension);
        kfree(deviceExtension);
        
		DEBUG("UpdateTimeStampThread: Thread exited... \n");
		
		up(&pkthread->semap);

} // UpdateTimeStampThread

/////////////////////////////////////////////////////////////////////
// GetRtcTime
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       GetRtcTime
//  Purpose:    Reads the RTC
//  Context:   
//  Returns:    Number of seconds since 1/1/1970 
//  Parameters:
//  Notes:      The muse: /linux-2.4.7-10/arch/i386/kernel/time.c 
				and /linux-2.4.7-10/driver/char/rtc.c
*F*/
unsigned long GetRtcTime(void)
{
        unsigned int year, mon, day, hour, min, sec, ctrl;
        int Saved70;
        unsigned long flags;
        spinlock_t RtcLock = SPIN_LOCK_UNLOCKED;
//      int i;

       // Disable Interrupts
        spin_lock_irqsave(&RtcLock, flags);
		// Save Value in port 0x70
        Saved70 = inb_p(RTC_PORT(0));       
        // Read the Clock
        sec =  CMOS_READ(RTC_SECONDS);
        min =  CMOS_READ(RTC_MINUTES);
        hour = CMOS_READ(RTC_HOURS);
        day =  CMOS_READ(RTC_DAY_OF_MONTH);
        mon =  CMOS_READ(RTC_MONTH);
        year = CMOS_READ(RTC_YEAR);
        ctrl = CMOS_READ(RTC_CONTROL);
        // Restore Value in port 0x70
        outb_p((Saved70), RTC_PORT(0));        
        // Enable interrupts 
        spin_unlock_irqrestore(&RtcLock, flags); 
        
        if (!(ctrl & RTC_DM_BINARY) || RTC_ALWAYS_BCD)
          {
            BCD_TO_BIN(sec);
            BCD_TO_BIN(min);
            BCD_TO_BIN(hour);
            BCD_TO_BIN(day);
            BCD_TO_BIN(mon);
            BCD_TO_BIN(year);
          }
        if ((year += 1900) < 1970)
                year += 100;
				
		return (mktime(year, mon, day, hour, min, sec));
} // GetRtcTime

/*F*
// ****************************************************************************
// * os_*_timer
// ****************************************************************************
//  Name:       
//  Purpose:    Envelope functions for the timer
//  Context:    
//  Returns:    
//  Parameters: 
//  Notes:      none
// ****************************************************************************
*F*/
static VOID OsTimerDpcFunction(unsigned long pTimer)
{
    os_timer_t * my_timer = (os_timer_t *) pTimer;

    if ( my_timer->timeOutFlag )
	    my_timer->timeOutFlag = 0;
}

void OsInitTimer(os_timer_t * pTimer, int milliseconds)
{
	init_timer( &pTimer->SmbTimer );
	pTimer->SmbTimer.data = (unsigned long)pTimer;
	pTimer->SmbTimer.function = OsTimerDpcFunction;
	pTimer->SmbTimer.expires = jiffies + ((milliseconds/1000) * HZ);
}

void OsSetTimer(os_timer_t * pTimer)
{
    pTimer->timeOutFlag = 1;
	add_timer(&pTimer->SmbTimer);
}

BOOL OsTimerTimedout(os_timer_t * pTimer)
{
	return (pTimer->timeOutFlag == 0);
}

void OsCancelTimer(os_timer_t * pTimer)
{
	del_timer(&(pTimer->SmbTimer));
}

void OsWaitForSemaphore(void * context)
{
    PDEVICE_EXTENSION    deviceExtension = (PDEVICE_EXTENSION) context;
    IMB_LOCK(deviceExtension->transactionSema);
    
}
void OsReleaseSemaphore(void * context)
{
    PDEVICE_EXTENSION    deviceExtension = (PDEVICE_EXTENSION) context;
    IMB_UNLOCK(deviceExtension->transactionSema);
    
}



