/*----------------------------------------------------------
* File:    imb_linmsgs.c
* Project: IMB Driver for Linux
*---------------------------------------------------------*/
/*----------------------------------------------------------
* Copyright (c) 2002-2006 Intel Corporation
*-----------------------------------------------------------*/
/*M*                     
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
*M*/
/*---------------------------------------------------------*/
/*
 * PVCS:
 * $Workfile:   imb_linmsgs.c  $ 
 * $Revision: 1.1 $
 * $Modtime:   22 Jul 2001 01:32:50  $
 * $Author: mks $
 * $Log: imb_linmsgs.c $
 * Revision 1.1  2002/12/11 06:12:04Z  mks
 * Initial revision
 * Revision 1.4  2002/12/08 12:25:18Z  marcela
 * Replace the linux71 flag by something more dynamic.
 * Revision 1.3  2002/12/01 15:12:30Z  marcela
 * Add SMBus
 * Revision 1.2  2002/11/12 13:56:54Z  marcela
 * Revision 1.1  2002/07/07 03:14:00Z  mks
 * Initial revision
 * 
 *    Rev 1.5   23 Jul 2001 11:17:34   spoola
 * Fixed tracker item #15462
 * 
 *    Rev 1.4   27 Jun 2001 10:58:06   spoola
 * Commented out DPC functionality for Linux 7.1, which is a temporary fix for imb driver crash
 * 
 *    Rev 1.3   04 Jun 2001 12:35:48   akumarx
 * GET_DEVICE_ID changes for IPMI 1.5 version
 * 
 *    Rev 1.2   May 23 2000 16:39:42   simont
 * 2.3 to 3.0 code merge.
 * 
 *    Rev 1.4   30 Dec 1999 14:00:46   mramacha
 * Changes to getbmcipmiversion similar to imb_nt.c
 * 
 *    Rev 1.3   29 Nov 1999 11:20:50   mramacha
 * Changes to getmbcversion to conform to
 * IPMI1.0 specs - similar to changes made
 * by pai to NT/UW/NW code.
 * 
 *    Rev 1.2   09 Sep 1999 13:37:30   mramacha
 * Changes to AsyncQSema and a few bug fixes.
 * 
 *    Rev 1.1   Aug 29 1999 20:47:46   mramachx
 * Various timer related changes to code.
 * 
 *    Rev 1.0   Aug 27 1999 17:58:22   mramachx
 * Initial Linux Checkin
 * 
 *
*/
/*
 * This file contains the OS layer functions that need to be exported
 * to the generic and interface specific layers of the imb driver.
 * The functions contained in this file are:
 * 		ImbOsStartReq
 * 		ImbOsEndReq
 * 		ImbAsyncOsInit
 *      ImbUnixAsyncInit --- for DPC
 *      ImbUnixAsyncLock --  for DPC
 *      ImbAsyncOsNewWakeup --  for DPC
 *      ImbUnixAsyncDeallocate - for DPC
 * 		ImbAsyncOsReqWait
 * 		AsyncOsTimerFunc
 * 		ImbAsyncOsReqWakeup
 * 		ImbAsyncOsLock
 * 		ImbAsyncOsUnLock
 * 		RequestProcessingThread
 * 		ImbOpenCloseDispatch
 * 		DelayRetryFunc
 * 		ImbReqTimedOut
 * 		GetNextImbSeq
 * 		ImbIf
 * 		TimerDpcFuncM
 * 		ImbAsyncQueue
 *		isIpmiOldVersion
 *		getBmcIpmiVersion
 *
*/


#include "imb_os.h"
#include "imb_drv.h"

///////////////////////////////////////////////////////////////////////////////
// IMB messaging support routines
///////////////////////////////////////////////////////////////////////////////
#define OBJECT_UNSIGNALLED 0
#define OBJECT_SIGNALLED 1
#define WAIT_TIMEDOUT 2
// needed only for AsyncMesgPoller: void AsyncMesgTimerFunc(PDEVICE_EXTENSION );
void AsyncOsTimerFunc( unsigned long);

void TimerDpcFuncM(unsigned long arg);

//
// Initialize and start the request timer, also, get the request semaphore 
// to avoid collisions.
//////////////////////////////////////////////////////////////////////
// ImbOsStartReq
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbOsStartReq
//  Purpose:    This function takes care of OS Specific Serialization
//              of BMC access and starts the Timeout timer.
//  Context:    called by functions which access the BMC
//  Returns:    void
//  Parameters:
//      timeOut Timeout interval to be set in uSec
//      context OS Specific context
//  Notes:      none
*F*/
void
ImbOsStartReq (
		DWORD	timeOut,	// in uSec units
		void *	context
	)
{
	PDEVICE_EXTENSION	deviceExtension = (PDEVICE_EXTENSION) context;
	unsigned long ticksM;
	
	DEBUG("ImbOsStartReq: timeout %ld devExt %p\n",timeOut,deviceExtension);

	if(deviceExtension->requestSemaphor == NULL)
	{
		IMB_LOCK_ALLOC(deviceExtension->requestSemaphor);
	}

	//
	// acquire the request semaphore
	//
	// IMB_LOCK(deviceExtension->requestSemaphor);
        down(&deviceExtension->imbreq_mutex);

	deviceExtension->timeOutFlagM = 1;
	
	//Set up the timer function to execute TimerDpcFuncM in timeOut Usec
	//ie, (usec/1,000,000)secs * HZ (ticksM/sec) => ticksM
	//from the current jiffies value. Same as timeout in Solaris.
	//Note I can use mod_timer instead, but this is same functionality. MR

	del_timer(&deviceExtension->mesgTimer);
	// ticksM = (unsigned long)((double)timeOut/1000000 * HZ);
	ticksM = (unsigned long)((timeOut *HZ)/1000000 );
	ticksM = (ticksM == 0)?1:ticksM;  //set it to atleast 1 tick

	DEBUG("ImbOsStartReq Timer to expire in %ld usec %ld ticks\n",timeOut, ticksM); 

	deviceExtension->mesgTimer.expires = jiffies + ticksM;
	add_timer(&deviceExtension->mesgTimer);

	// We are not releasing the lock now, we wait till ImbOsEndReq
	// to do that. 
	
	DEBUG("ImbOsStartReq: timeout set\n");
}


//
// Cancel a pending request timeout and release the request semaphore
//
//
//////////////////////////////////////////////////////////////////////
// ImbOsEndReq
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbOsEndReq
//  Purpose:    This function takes care of OS Specific Serialization
//              of BMC access and kills the Timeout timer.
//  Context:    called by functions which access the BMC
//  Returns:    void
//  Parameters:
//      context OS Specific context
//  Notes:      none
*F*/
void
ImbOsEndReq (
		void *	context
	)
{
	PDEVICE_EXTENSION	deviceExtension = (PDEVICE_EXTENSION) context;

	DEBUG("ImbOsEndReq: deviceExtension %p\n", deviceExtension);

	if ( deviceExtension->timeOutFlagM ) {
		del_timer( &deviceExtension->mesgTimer);
		deviceExtension->timeOutFlagM = 0;
	}
	
	// IMB_UNLOCK(deviceExtension->requestSemaphor);
        up(&deviceExtension->imbreq_mutex);
}


//////////////////////////////////////////////////////////////////////
// ImbAsyncOsInit
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsInit
//  Purpose:    This function initializes the OS dependent parts of the
//             Async Queue
//  Context:    called by Async Initialization functions
//  Returns:    void
//  Parameters:
//     asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
void
ImbAsyncOsInit ( ImbAsyncQueueInfo *asyncQ)
{
	//
	// init the semaphore
	//
	DEBUG("ImbAsyncOsInit: Set up locks and CV for Async Q\n");

	IMB_LOCK_ALLOC(asyncQ->syncSem);
	IMB_LOCK_ALLOC(asyncQ->waitEvent.plock);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  asyncQ->waitEvent.waitOn = NULL;
#else
  init_waitqueue_head(&asyncQ->waitEvent.waitOn);
#endif
	
	//
	//Initialize the asyncMesgTimer 
	//
	DEBUG("ImbAsyncOsInit: initialize asyncMesgTimer\n");
	init_timer( &asyncQ->waitEvent.asyncMesgTimer);
	asyncQ->waitEvent.asyncMesgTimer.data = (unsigned long)asyncQ;
	asyncQ->waitEvent.asyncMesgTimer.function = AsyncOsTimerFunc ;

	asyncQ->waitEvent.state=OBJECT_UNSIGNALLED;
	
} //ImbAsyncOsInit

/************************************************************************/
/* The next 4 functions were added for the  DPC support. These are similar
 * to the code you will find in imb_unix.c and imb_nt.c done by Pai.
 * mahendra 7/99.
 */
/************************************************************************/

//////////////////////////////////////////////////////////////////////
// ImbUnixAsyncInit
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbUnixAsyncInit
//  Purpose:    This function initializes the synchronization var associated
//              with the DPC thread. This is different from the asyncQ referred
//              to in the ImbAsyncOsInit function above which is used in
//              imb_asyn.c functions.
//  Context:    called as a result of the  IOCTL_IMB_REGISTER_ASYNC_OBJ
//              which is called by the dpc thread at user-level.
//  Returns:    pointer to the allocated mutex or 0 on failure.
//  Parameters:
//     deviceExtension
//  Notes:      none
*F*/
int
ImbUnixAsyncInit (
				PDEVICE_EXTENSION deviceExtension
                  )

{
    DEBUG("imb: ImbUnixAsyncInit called\n");

	//Note that there really isn't anything for us to setup in Linux for
	//this ioctl. The lock,unlock and deallocate are what are necessary.
	//What we can do here is sanity check that we do not get here after
	//a call to ImbUnixAsyncLock has been called.

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
	if (deviceExtension->asyncQSema)
		return (0);
	else
    	return (1);
#else
    return (1);
#endif    

}


//////////////////////////////////////////////////////////////////////
// ImbUnixAsyncLock
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbUnixAsyncLock
//  Purpose:    This function grabs the lock associated with the DPC thread.
//  Context:    called as a result of the  IOCTL_IMB_CHECK_EVENT
//              which is called by the dpc thread at user-level.
//  Returns:    void
//  Parameters:
//     deviceExtension
//  Notes:      none
*F*/

void
ImbUnixAsyncLock (
                  PDEVICE_EXTENSION deviceExtension

            )
{
    DEBUG("imb: ImbUnixAsyncLock called\n");

    /* safety check. make sure we have not de-allocated this as a result of
     * closing the device (basebrd died) or due to a
     * IOCTL_IMB_DEREGISTER_ASYNC_OBJ.
     */

	//    if (deviceExtension->asyncQSema)
    	 OS_interruptible_sleep_on(&deviceExtension->asyncQSema);
        /* deprectated, migrate to wait_event_interruptible() */ 

}

//////////////////////////////////////////////////////////////////////
// ImbAsyncOsNewWakeup
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsNewWakeup
//  Purpose:	unlock the asyncQSema.
//  Context:	called by ImbAsyncQueueMessage.
//  Returns:    void
//  Parameters:
//     deviceExtension
//  Notes:
//      NEW async wakeup 
*F*/
void ImbAsyncOsNewWakeup(void *context)
{
    PDEVICE_EXTENSION   deviceExtension = (PDEVICE_EXTENSION) context;

    DEBUG("imb: ImbAsyncOsNewWakeup called\n");
	
	/*saftey check. make sure we have not de-allocated this as a result of
     *closing the device because the basebrd app died or due to a
     *IOCTL_IMB_DEREGISTER_ASYNC_OBJ.
     */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
    if (deviceExtension->asyncQSema)
    	OS_wake_up_interruptible(&deviceExtension->asyncQSema);
#else
	OS_wake_up_interruptible(&deviceExtension->asyncQSema);
#endif
}


//////////////////////////////////////////////////////////////////////
// ImbUnixAsyncDeallocate
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbUnixAsyncDeallocate
//  Purpose:	de-allocate the aysncQSema.
//  Context:	called either due to IOCTL_IMB_DEREGISTER_ASYNC_OBJ
//              or in imb_close when basebrd goes away.
//  Returns:    void
//  Parameters:
//     deviceExtension
//  Notes:
//			None
*F*/

void ImbUnixAsyncDeallocate(void *context)
{
    PDEVICE_EXTENSION   deviceExtension = (PDEVICE_EXTENSION) context;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
	if (deviceExtension->asyncQSema) {
    	DEBUG("imb: ImbUnixAsyncDeallocate waking asyncQSema\n");
    	OS_wake_up_interruptible(&deviceExtension->asyncQSema); 
		deviceExtension->asyncQSema = NULL;
		return;
	}
#else
    OS_wake_up_interruptible(&deviceExtension->asyncQSema); 
#endif

    DEBUG("imb: ImbUnixAsyncDeallocate nothing to free\n");
}

/************************************************************************/
// End of DPC related functionality.
/************************************************************************/


//
// wait for a new SMS message to come in
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncOsReqWait
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsReqWait
//  Purpose:    This function makes the incoming thread wait for a new
//             SMS message to come in
//  Context:    called by Async functions
//  Returns:    void
//  Parameters:
//     asyncQ  pointer to Async Queue Specific Info
//     timeOut Time to wait before timing out.
//  Notes:      none
*F*/
void
ImbAsyncOsReqWait (
		ImbAsyncQueueInfo *	asyncQ, 
		DWORD				timeOut
	)
{
	// NTSTATUS		status;
	unsigned long ticksM;

	DEBUG("ImbAsyncOsReqWait\n");

	/* clear event */
	IMB_LOCK(asyncQ->waitEvent.plock);

	asyncQ->waitEvent.state=OBJECT_UNSIGNALLED;

    IMB_UNLOCK(asyncQ->waitEvent.plock);

	//First reset the timer incase it is active.
	del_timer( &asyncQ->waitEvent.asyncMesgTimer);

	//Set up the timer function to execute AsyncOsTimerFunc in timeOut Usec
	//ie, (usec/1,000,000)secs * HZ (ticksM/sec) => ticksM
	//from the current jiffies value.

	// ticksM = (unsigned long)((double)timeOut/1000000 * HZ);
	ticksM = (unsigned long)((timeOut *HZ)/1000000 );
	ticksM =  (ticksM == 0) ? 1 : ticksM; //set it to atleast one tick.

	DEBUG("%s: AsyncOsTimerFunc expires in %ld usec %ld ticksM\n",
				__FUNCTION__,timeOut, ticksM); 

	asyncQ->waitEvent.asyncMesgTimer.expires = jiffies + ticksM;
	add_timer( &asyncQ->waitEvent.asyncMesgTimer);

	//Do we need to check if a prior call to this function has already
	//blocked a process? Mahendra

	//block the calling process on the asyncQ's waitEvent
	OS_interruptible_sleep_on(&asyncQ->waitEvent.waitOn);
        /* deprectated, migrate to wait_event_interruptible() */
} 


//////////////////////////////////////////////////////////////////////
// AsyncOsTimerFunc
//////////////////////////////////////////////////////////////////////
// Name:	AsyncOsTimerFunc
// Purpose:	???
// Context:	???
// Reutrns: ???
// Parameters:
// Notes:
//////////////////////////////////////////////////////////////////////
void
AsyncOsTimerFunc(unsigned long arg)
{
	ImbAsyncQueueInfo *asyncQ = (ImbAsyncQueueInfo *)arg;

	DEBUG("AsyncOsTimerFunc WAIT_TIMEDOUT\n");

	IMB_LOCK(asyncQ->waitEvent.plock);

	asyncQ->waitEvent.state=WAIT_TIMEDOUT;

	IMB_UNLOCK(asyncQ->waitEvent.plock);
	OS_wake_up_interruptible(&asyncQ->waitEvent.waitOn);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
	asyncQ->waitEvent.waitOn = NULL;
#endif

}


// 
// wake all sleepers waiting for a message to come in
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncOsReqWakeup
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsReqWakeup
//  Purpose:    This function wakes up all sleeping threads waiting for a
//             message to come in
//  Context:    called by Async  functions
//  Returns:    void
//  Parameters:
//     asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
void
ImbAsyncOsReqWakeup (
		ImbAsyncQueueInfo *	asyncQ
	)
{
	DEBUG("ImbAsyncOsReqWakeup\n");

	IMB_LOCK(asyncQ->waitEvent.plock);

	asyncQ->waitEvent.state=OBJECT_SIGNALLED;

	IMB_UNLOCK(asyncQ->waitEvent.plock);
	OS_wake_up_interruptible(&asyncQ->waitEvent.waitOn);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
	asyncQ->waitEvent.waitOn = NULL;
#endif

}


//
// Get exclusive use of the async message queue
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncOsLock
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsLock
//  Purpose:    This function obtains exclusive use of the async message
//             queue
//  Context:    called by Async  functions
//  Returns:    returns true if able to obtain exclusive use of Async Q
//  Parameters:
//     asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
BOOL
ImbAsyncOsLock (
		ImbAsyncQueueInfo *asyncQ
	)
{
	DEBUG("ImbAsyncOsLock\n");
	IMB_LOCK(asyncQ->syncSem);

	// Grabbed the lock so now return TRUE to caller
	return TRUE;
}


//
// Release our lock on the async message queue
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncOsUnLock
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncOsUnLock
//  Purpose:    This function releases exclusive use of the async message
//             queue
//  Context:    called by Async  functions
//  Returns:    returns true if able to release exclusive use of Async Q
//  Parameters:
//     asyncQ  pointer to Async Queue Specific Info
//  Notes:      none
*F*/
BOOL
ImbAsyncOsUnLock (
		ImbAsyncQueueInfo *asyncQ
	)
{
	DEBUG("ImbAsyncOsUnLock\n");

	IMB_UNLOCK(asyncQ->syncSem);

	return TRUE;
}

/* this function checks if there is a message in 
the sms buffer, it copies the message and reschedules it
by calling timeout, if there are no messages reschedules and sleeps */

#ifndef LINUX
// Do we need this for solaris or Linux ?
//void
AsyncMesgPoller(PDEVICE_EXTENSION deviceExtension)
{
//	clock_t ticks;
	// SMS_CHECK_PERIOD is in millisec, convert microsec
//	ticks  = drv_usectohz(SMS_CHECK_PERIOD * 1000);
//	while(1)
//	{
//		if ( deviceExtension->unloadingDriver )
//		{
//			/* might have to release all the locks ?????????? */
//			break;
//		}
//		ImbAsyncMessageProcess ( deviceExtension );
//		timeout(AsyncMesgTimerFunc, deviceExtension, ticks);
//		/* sleep on any var , instead of creating a new var
//		will use deviceExtension itself */
//		sleep(deviceExtension, pltimeout);
//	}
//
//
}
//void
AsyncMesgTimerFunc(PDEVICE_EXTENSION deviceExtension)
{
//	wakeup(deviceExtension);
//
}
#endif //LINUX

//////////////////////////////////////////////////////////////////////
// DelayRetryFunc
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       DelayRetryFunc
//  Purpose:    This function inserts a delay into the execution of
//              current thread
//  Context:    called by functions which access the BMC
//  Returns:    void
//  Parameters:
//      retryCount Number of times to be spin before sleeping
//      delayTime Delay interval to be used in uSec
//      context OS Specific context
//  Notes:      none
*F*/
VOID
DelayRetryFunc (
        DWORD retryCount,   // number of times we've been called, not used here
        DWORD delayTime,    // time to delay (in uSecs)
        void *context       // OS dependent handle,  not used here
    )
{
      static DWORD maxSpin = 0;

	if( retryCount < NO_RETRY_DELAY_COUNT && retryCount != FORCE_DELAY ) {
		// DEBUG("imb DelayRetryFunc(%d): no delay imposed\n",maxSpin);
                maxSpin++;
		return;
	} else maxSpin = 0;
        DEBUG("imb DelayRetryFunc(retrys=%ld,delay=%ld)\n",retryCount,delayTime);

	// Delay - There's a new envelope for the delay. Now it calls
	// udelay or mdelay - it depends on the delaytime value.
	Delay(delayTime);
}

//
// This function tells OS independent code if the request timer for the current
// in-progress request has expired
//
//////////////////////////////////////////////////////////////////////
// ImbReqTimedOut
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbReqTimedOut
//  Purpose:    This function determines if the Timeout Timer has
//              timed out
//  Context:    called by functions which use the Timeout Timer.
//  Returns:    This function returns TRUE if the Timeout has expired
//              else returns FALSE.
//  Parameters:
//      context OS Specific context
//  Notes:      none
*F*/
extern int imb_has_paniced;
BOOL
ImbReqTimedOut( void *context )
{
// This debug stmt is too painful since it is call very often.
//	DEBUG("ImbReqTimedOut %s\n",
//		(((PDEVICE_EXTENSION) context)->timeOutFlagM == 0)? "True": "False");

    if (imb_has_paniced) return 0;
    return( ((PDEVICE_EXTENSION) context)->timeOutFlagM == 0 );
}

//////////////////////////////////////////////////////////////////////
//
// Return the next sequence number to be used when sending an IMB message.
// Although the IMB sequence number is only a 6 bit field, we ignore that
// and treat it as a BYTE. The IMB code wil mask it appropriately.
//
//////////////////////////////////////////////////////////////////////
// GetNextImbSeq
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       GetNextImbSeq
//  Purpose:    This function gets the next Imb Sequence number
//  Context:    called by functions sending IMB commands to devices
//  Returns:    This function returns the next possible sequence number
//  Parameters:
//      context OS Specific context
//  Notes:      The Sequence number takes values from 0-255
*F*/
BYTE
GetNextImbSeq (
        void *context
    )
{
	DEBUG("GetNextImbSeq\n");
    return( ((PDEVICE_EXTENSION) context)->imbSeqNo++ );
}

//
// Return a pointer to the IMB interface's function vector
//
//////////////////////////////////////////////////////////////////////
// ImbIf
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbIf
//  Purpose:    This function returns a pointer to the currently
//              detected Hardware Interface
//  Context:    called by  routines Interfacing to the SMIC or KCS layer
//  Returns:    Returns a pointer to the currently detected Hardware
//              Interface
//  Parameters:
//      context OS Specific context
//  Notes:
*F*/
ImbInterface *
ImbIf(
        void *context
    )
{
    return ((PDEVICE_EXTENSION) context)->imbIf;
}

//////////////////////////////////////////////////////////////////////
// Name: TimerDpcFuncM
//
//////////////////////////////////////////////////////////////////////
//
// Timer callback function used with the mesgTimer field.
// Name:	TimerDpcFuncM
// Purpose: Timer callback function used with the mesgTimer field.
// Context:	setup by the ImbOsStartReq function
// Parameters:	DeviceExtension structure.
// Notes:	None.
//
void
TimerDpcFuncM(unsigned long arg)
{
    PDEVICE_EXTENSION deviceExtension = (PDEVICE_EXTENSION)arg;

	DEBUG("IMB: TimerDpcFuncM went off\n");

	if (deviceExtension == NULL) {	//Something is very wrong here
		DEBUG(KERN_ERR "ISC: TimerDpcFuncM arg is NULL\n");
		return;
	}

    if ( deviceExtension->timeOutFlagM )
        deviceExtension->timeOutFlagM = 0;
	// timer fired, no need to delete or reset mesgTimer
}

///////////////////////////////////////////////////////////////////////////
//
// Return a pointer to the Async queue info
//
//////////////////////////////////////////////////////////////////////
// ImbAsyncQueue
/////////////////////////////////////////////////////////////////////
/*F*
//  Name:       ImbAsyncQueue
//  Purpose:    This function returns a pointer to the Async queue info
//  Context:    called by functions manipulating the Async Q
//  Returns:    returns a pointer to the Async queue info
//  Parameters:
//      context OS Specific context
//  Notes:      none
*F*/
ImbAsyncQueueInfo *
ImbAsyncQueue (
        void *context
    )
{
	DEBUG("ImbAsyncQueue\n");
    return( &(((PDEVICE_EXTENSION)context)->asyncQ) );
}

/////////////////////////////////////////////////////////////////////////
// isIpmiOldVersion
////////////////////////////////////////////////////////////////////////
/*F*
//  Name: isIpmiOldVersion
//  Purpose: Check if the bmcVer field of the deviceExtension structure is
//           set to IPMI_OLD_VER or IPMI_1_VER.
//  Context: called by the functions in imb_drv.c to send properly formated
//           messages to the BMC.
//  Returns: 1 if v0.9 and 0 otherwise.
//  Parameter:  context of OS specific state structure
//  Notes:  None.
*F*/

int
isIpmiOldVersion (void *context)
{
    PDEVICE_EXTENSION deviceExtension = (PDEVICE_EXTENSION) context;

    DEBUG("imb: isIpmiOldVersion bmcVer = %x\n", deviceExtension->bmcVer);
    if (deviceExtension->bmcVer == (UCHAR)IPMI_OLD_VER)
        return 1;
    else
        return 0;
}


/////////////////////////////////////////////////////////////////////////
// getBmcIpmiVersion
////////////////////////////////////////////////////////////////////////
/*F*
//  Name: getBmcIpmiVersion
//  Purpose: Detect whether we are dealing with IPMIv0.9 or v1.0 FW
//  Context: Used in driverentry to set the version correctly.
//  Returns: Nothing.
//  Parameter:  context of OS specific state structure - bmcVer in this
//              structure is set accordingly.
//  Notes:
//      According to Pai 1/13/99:
//  Get the IPMI version of the BMC controller. The only way to check it is by
//  looking at the response length and byte 6 of the response data of the BMC
//  command 'GetDeviceId'. This method defeats the purpose of the IMB driver
//  being just a communication pipe since now it has to be made aware of the
//  firmware functionality.   but we do not have a choice here.
//
*F*/
VOID getBmcIpmiVersion( PDEVICE_EXTENSION deviceExtension)
{
    ImbRequestBuffer * pimbReq;
    ImbResponseBuffer * pimbResp;
    unsigned char reqbuf[255];      //Pai allocates 255 bytes for these two
    unsigned char respbuf[255];     //structures in the imb_nt.c code.
                                    //I am allocating it on the stack instead of
                                    //from the heap. mahendra.
    DWORD respLength = MAX_BMC_RESPONSE_SIZE;
    static int bmcVer = IPMI_UNKNOWN_VER;

    DEBUG("imb: getBmcIpmiVersion probing the f/w\n");

    if (bmcVer != IPMI_UNKNOWN_VER) {
       deviceExtension->bmcVer = bmcVer;
       return; 
    }
    pimbReq = (ImbRequestBuffer *)reqbuf;
    pimbResp = (ImbResponseBuffer *)respbuf;

    pimbReq->req.rsSa            = BMC_SA;
    pimbReq->req.cmd             = GET_DEVICE_ID;
    pimbReq->req.netFn           = APP_NETFN;
    pimbReq->req.rsLun           = 0;
    pimbReq->req.dataLength      = 0;
    pimbReq->flags               = 0;
    pimbReq->timeOut             = DEFAULT_MESSAGE_TIMEOUT;

    ImbOsStartReq(DEFAULT_MESSAGE_TIMEOUT, deviceExtension );

	//Made changes here similar to what Pai had to do in the NT driver.
	//First we fake the BMC version as IPMI 0.9 and if that fails try with
	//faking a 1.0. mahendra
    deviceExtension->bmcVer = IPMI_OLD_VER;

    if ( ( SendImbRequest( pimbReq,
                        pimbResp,
                        & respLength,
                        deviceExtension
                    ) == IMB_SUCCESS ) &&
					(!pimbResp->cCode) &&
					(respLength < (IPMI10_GET_DEVICE_ID_RESP_LENGTH-1))
        )
    {
		// so it is the old IPMI version
		;
	}
	else {
		ImbOsEndReq(deviceExtension);

		//Didn't work. Now try with 1.0
 
		deviceExtension->bmcVer = IPMI_1_VER;
		respLength = MAX_BMC_RESPONSE_SIZE;
		ImbOsStartReq(DEFAULT_MESSAGE_TIMEOUT, deviceExtension );

		if ( SendImbRequest( pimbReq,
							pimbResp,
							&respLength,
							deviceExtension
						) != IMB_SUCCESS
			)
		{
			//This is not good at all. No success with either 1.0 or 0.9.
			DEBUG("imb: getBmcIpmiVersion - IPMI_UNKNOWN_VER\n");
			deviceExtension->bmcVer = (UCHAR) IPMI_UNKNOWN_VER;	
		}
		else {
			// New version of IPMI?
			// check Byte 5 in the response data to determine the IPMI version.

    		if ( (!pimbResp->cCode) &&
         	   ( respLength >= (IPMI10_GET_DEVICE_ID_RESP_LENGTH-1)))
			{
				switch (pimbResp->data[4])
				{
				case IPMI_1_VER:
					deviceExtension->bmcVer = IPMI_1_VER;
					break;
					//Add new versions here later.

				case IPMI_15_VER:

					deviceExtension->bmcVer = (UCHAR)IPMI_15_VER;
					break;

				default:
					//default right now assume the ipmi version to be 1.5
					deviceExtension->bmcVer = (UCHAR)IPMI_15_VER;							
					break;
				}
			}
			else
				deviceExtension->bmcVer = IPMI_UNKNOWN_VER;
		}
	}
		

	ImbOsEndReq(deviceExtension);
    DEBUG("imb: getBmcIpmiVersion IPMI is %x\n", deviceExtension->bmcVer);
    bmcVer = deviceExtension->bmcVer;

}


