/*----------------------------------------------------------
* File Contents:
*     imb_drv.c
* Project: IMB Driver for Linux
*-----------------------------------------------------------
* Copyright (c) 2002-2006 Intel Corporation 
*---------------------------------------------------------*/
/*M*                     
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
*M*/
/*---------------------------------------------------------*/
/*
 *	$Workfile:   imb_os.h  $
 *	$Revision: 1.2 $
 *	$Modtime:   22 Jul 2001 01:29:12  $
 *	$Author: marcela $
 *	$Log: imb_os.h $
 *	Revision 1.2  2003/04/01 14:21:54Z  marcela
 *	add semaphore
 *	Revision 1.1  2002/12/11 06:12:04Z  mks
 *	Initial revision
 *	Revision 1.6  2002/12/10 12:18:38Z  marcela
 *	Add function headers
 *	Revision 1.5  2002/12/09 12:52:28Z  marcela
 *	Replace the linux71 flag by something more dynamic
 *	Revision 1.4  2002/12/02 12:51:06Z  marcela
 *	Revision 1.3  2002/12/01 14:33:20Z  marcela
 *	Revision 1.2  2002/11/12 13:56:54Z  marcela
 *	Revision 1.1  2002/07/07 03:14:02Z  mks
 *	Initial revision
 * 
 *    Rev 1.4   23 Jul 2001 11:16:34   spoola
 * Fixed tracker item #15462
 * 
 *    Rev 1.3   27 Jun 2001 08:44:44   spoola
 * Fixes made for linux7.1
 * 
 *    Rev 1.2   Jun 02 2000 10:02:26   simont
 * Ported 2.4 changed.
 * 
 *    Rev 1.3   24 Apr 2000 16:55:36   mramacha
 * Changed the include file from ntddk to windefs.
 * 
 *    Rev 1.2   09 Sep 1999 13:38:54   mramacha
 * Linux OS dependent changes
 * 
 *    Rev 1.1   Aug 29 1999 20:55:06   mramachx
 * Various code changes have gone in here.
 * 
 *    Rev 1.0   Aug 27 1999 17:10:56   mramachx
 * Initial Linux Checkin
 * 
*M*/

/*
 * This code is based on the following two files:
 * 
 * This file from ISC1.10 Solaris work:
 *
 * cospublic004/shareddata/isc/iscdev/solaris/src/imb/imb_os.h_v
 *    Rev 1.0   Apr 21 1999 13:29:28   mramachX
 * Initial Checkin for Solaris
 *
 * And this file from 2.1:
 *
 * \\cospublic004\shareddata\isc\ISC2x\ntwork\src\imb\imb_os.h.v
 *
 *    Rev 1.3   May 07 1999 11:56:02   sganeshx
 * 1. added a new #define for response length
 *     to the IPMI 1.0 Get Device ID command
 * 2. modified the check in the GetDevice ID
 *    function to check for
 * (IPMI10_GET_DEVICE_ID_RESP_LENGTH-1)
 *
 *    Rev 1.2   18 Mar 1999 13:43:58   sganeshx
 * added 'bmcVer' field to DEVICE_EXTENSION
 *
 *    Rev 1.1   08 Dec 1998 10:55:48   jpaiasno
 * Added code to support async.
 * DPC events from the EMP console.
 *
 *    Rev 1.8   24 Mar 1998 14:24:42   fkhanx
 * Added Copyright Header
*/


//
// This file exports the os specific typedefs, macro implementations and
// structure definitions to all the other files in imb. This allows the
// for abstraction necessary to implement the os-independent code.
//
// This is derived from the Solaris version of the OS specific header file
// but is Linux specific.
//


//#include <sys/stat.h>
//#include <sys/mman.h>

/* Note this should only be in imb_lin.c:  <linux/module.h> to prevent linking
 * errors. MR
 */

// For the Linux Version 
#define __NO_VERSION__
#include <linux/module.h>
#include <linux/version.h>

#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/delay.h>
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,7))
	#include <linux/malloc.h>
#else
	#include <linux/slab.h>
#endif		
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  #include <asm/spinlock.h>
#else
  #include <linux/spinlock.h>
#endif
#include <linux/errno.h>
#include <linux/param.h>


#include <linux/mc146818rtc.h> //RTC reading done in Update Timestamp Thread

#define  OS_interruptible_sleep_on 			interruptible_sleep_on
#define  OS_interruptible_sleep_on_timeout 	interruptible_sleep_on_timeout
#define  OS_wake_up_interruptible  			wake_up_interruptible
#define  OS_kernel_thread					kernel_thread

#include "windefs.h"
#include "imb_log.h"
#include "smi.h"


#define False	0
#define True	1

///////////////////////////////////////////////////////////////////
// Macros used by imb_lin and imb_linmsgs
///////////////////////////////////////////////////////////////////
#ifdef LINUX_DEBUG
#define DEBUG(format, args...) printk(format, ##args)
#else
#define DEBUG(format, args...)
#endif


//
// Defines for IMB_STATUS values
//
typedef NTSTATUS IMB_STATUS; 

#define IMB_SUCCESS					STATUS_SUCCESS
#define DUMP_COUNT					4

//
// From ncr - just to get the imbprobe compiling
#define MAX_WAIT_RETRIES  200
#define SMS_CC_GET_STATUS 0x40
#define BUSY_FLAG         1
#define SMS_SC_READY      0x0c0
#define MEMORY_SPACE      0



// Timer conversion constant 100 nanoseconds to seconds
//
#define TIMER_DELAY_USEC		(ULONG)(-10)
#define TIMER_DELAY_MSEC		(-10 * 1000)
#define TIMER_DELAY_SECOND		(ULONG)(-10 * 1000 * 1000)

// Timer conversion macro (t in microseconds) into clock ticks
#define TICKS(t)                (drv_usectohz(t / 1000))


//
// SMS buffer polling period - 100 mSec
//
#define SMS_CHECK_PERIOD		(100 * TIMER_DELAY_MSEC)

//
// Update Time Stamp = 0.5 hour = 30 * 60 seconds = 1800  seconds
//
#define UPDATE_TIMESTAMP         HZ * 1800

//
// error logging related definitions
//
#define IMB_ERROR_VALUE_BASE	0x00008060

// value for IPMI 1.0 Get Device ID response length 
#define IPMI10_GET_DEVICE_ID_RESP_LENGTH 12

//
// Macros for I/O port access. Used by kcs.c and smic.c to talk to the hardware
//

#define ReadPortUchar( addr, valp ) (*(valp) = inb( (addr) ) )
#define WritePortUchar( addr, val ) ( outb( (val),  (addr) ) )
#define ReadPortUlong( addr, valp ) (*(valp) = inl( (addr) ) )
#define WritePortUlong( addr, val ) ( outl( (val),  (addr) ) )

//
//the ones above work fine but the ones below don't, I think. MR
//#define ReadPortUchar( addr, valp ) (*(valp) = readb( (addr) ) )
//#define WritePortUchar( addr, val ) ( writeb( (val),  (addr) ) )

//
// To make copying and zeroing of memory generic  and thus available to
// the generic imb code, all OSes provide these macros. 
//
#define CopyMemory( dst, src, count )	memcpy( (dst), (src), (count) )
#define ZeroMemory( dst, count )	memset( (dst), 0, (count) )


//
//locking macros
//
#ifndef LINUX_DEBUG

#define IMB_LOCK_ALLOC(me) \
			   (me) = kmalloc(sizeof(spinlock_t), GFP_KERNEL); \
               spin_lock_init((me))

#define IMB_LOCK_DEALLOC(me) kfree((me))

#define IMB_LOCK(me)     spin_lock((me))

#define IMB_UNLOCK(me)   spin_unlock((me))

#else //ie debug version

#define IMB_LOCK_ALLOC(me)  \
				printk("%s calls IMB_LOCK_ALLOC\n", __FUNCTION__);\
				(me) = kmalloc(sizeof(spinlock_t), GFP_KERNEL); \
                spin_lock_init((me))

#define IMB_LOCK_DEALLOC(me)  \
				printk("%s calls IMB_LOCK_DEALLOC\n", __FUNCTION__);\
				kfree((me))

#define IMB_LOCK(me)  \
			    printk("%s calls IMB_LOCK\n", __FUNCTION__);\
				spin_lock((me));\
			    printk("%s IMB_LOCK held\n", __FUNCTION__)

#define IMB_UNLOCK(me)  \
				printk("%s calls IMB_UNLOCK\n", __FUNCTION__);\
				spin_unlock((me))

#endif //LINUX_DEBUG

//
//
// Async message related types
//

typedef spinlock_t *ImbAsyncOsSem;

typedef struct //event_t
{
	int		state;
	spinlock_t *plock;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  struct wait_queue *	waitOn;
#else
  wait_queue_head_t waitOn;
#endif


	struct timer_list asyncMesgTimer;
} ImbAsyncOsEvent;


/*****************end of definitions *******************************/

#include "imb_if.h"

#include "imb_asyn.h"

//Mahendra added this Pai addition:

//maximum number of client applications that can register for async
//event notification

#define MAX_EVENT_OBJ_NUMBER  5

///////////////////////////////////////////////////////////////////////////
// DEVICE_EXTENSION
///////////////////////////////////////////////////////////////////////////

/*D*
//  Name:       DEVICE_EXTENSION
//  Purpose:    structure to hold the Imb driver's device extension.
//  Context:    Used by the I/O Manager to call the driver routines
//	Notes:		This name has been kept for consistency with NT
//				and UW, although the fields may be different.
//
*D*/

typedef struct  _DEVICE_EXTENSION 
{
	spinlock_t  		*requestSemaphor;	// Synchronization in Linux
      spinlock_t  		*transactionSema;	

	// keep till we decide whether or not to support AsyncMesgPoller() - MR
	UCHAR       		unloadingDriver;	//default 0. Set to 1 
											//during driver unload 

	struct timer_list	mesgTimer;		//request timeout generating timer 
	UCHAR				timeOutFlagM;	// IMB request related timeout 
										//flag. Set in DPC (M). 	
    UCHAR				shutdownCode;	//code for shut down action 
	int					delayTime;		//delay before shutdown, in 100ms units
	UCHAR				imbSeqNo;		//next sequence number to use 
										//when sending an IMB request
	struct ImbIfStruct	*imbIf;			//pointer to vector of routines 
										//to access IMB interface
	ImbAsyncQueueInfo	asyncQ;			//holds async queue related state

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
  struct wait_queue *	asyncQSema;
#else
  wait_queue_head_t asyncQSema;
#endif


   unsigned char 		bmcVer;         // IPMI Version

   struct semaphore imbreq_mutex;

}DEVICE_EXTENSION, *PDEVICE_EXTENSION;

///////////////////////////////////////////////////////////////////////////
// KERNEL THREAD STRUCT
///////////////////////////////////////////////////////////////////////////

/*D*
//  Name:       kthread_struct
//  Purpose:    structure to hold the Update Time Stamp Thread data.
//  Context:    Used by ImbUpdateTimeStamp Thread 
//	Notes:		
//
*D*/

typedef struct kthread_struct
{
	// Wait Queue - for the 1/2 hour timeout
	#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
		  struct wait_queue *	queue;
	#else
		  wait_queue_head_t queue;
	#endif

    // flag - die or don't 
	int terminate;	
	// Any Failure?
	int threadFail;	
	// semaphore - creation/destruction of thread
    struct semaphore   semap;
     
} kthread_t;

///////////////////////////////////////////////////////////////////////////
// OS DEPENDENT TIMER - STRUCT + FUNCTIONS
///////////////////////////////////////////////////////////////////////////

/*D*
//  Name:       os_timer_t
//  Purpose:    
//  Context:    
//	Notes:		
//
*D*/

typedef struct 
{
	struct timer_list	SmbTimer;		//timer for smbus transactions 
	UCHAR				timeOutFlag;	// smbus transactions timeout 		
} os_timer_t;

void OsInitTimer(os_timer_t * pTimer, int milliseconds);
void OsSetTimer(os_timer_t * pTimer);
BOOL OsTimerTimedout(os_timer_t * pTimer);
void OsCancelTimer(os_timer_t * pTimer);
