/*----------------------------------------------------------
* File Contents:
*     pci.c
* Project: IMB Driver
*-----------------------------------------------------------
* Copyright (c) 2003-2007 Intel Corporation
*---------------------------------------------------------*/
/*M*                     
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
*M*/
/*---------------------------------------------------------*/
/*
 * $Log: pci.c $
 * Revision 1.5  2006/03/31      Andy Cress
 * only probe certain pci addresses
 *               2003/11/22 assigned to Intel
 * Revision 1.4  2003/03/12 12:05:28Z  marcela
 * Add AMD
 * Revision 1.3  2003/01/29 10:44:48Z  marcela
 * Add ICH5
 * Revision 1.2  2002/12/17 13:34:56Z  marcela
 * Add (()) to DEBUG expressions
 * Revision 1.1  2002/12/11 06:54:16Z  mks
 * Initial revision
 */

#if defined(WIN32) || defined(WINVER)
	#include <wdm.h>
#endif
#include "imb_os.h"
#include "pci.h"

#define NPCILIST  1
static struct {
  UCHAR dev;
  UCHAR func;
  } pcilist[NPCILIST] = {
  { 31, 3 }   /* SMBus for Intel STP, SJR boards */
};
// 03/31/2006  Andy Cress
// Avoid probing delicate PCI devices like DIMMs and plugin adapters,
// since Write/Read probing to these is damaging.
// Use the pcilist array of known SMBus locations instead of every
// device from 0 to 31, and every func from 0 to 7.

//
// SearchPci()
//
BOOL PciFind (PULONG ID, PUINT pdev, PUINT pfunc)
{
    ULONG   pci_data;
    BOOL    found = FALSE;
    UCHAR   i;
#ifdef OLD
    UCHAR  i, j;
    
    // Loop in order to find PCI device
    for( i=31 ; i>=0 ; i-- ){
        for( j=0 ; j<8 ; j++ ){
            pci_data = GetPciDevId( i, j );
#endif
    for (i = 0; i < NPCILIST; i++ ) {
            pci_data = GetPciDevId( pcilist[i].dev, pcilist[i].func );
	    if (pci_data != 0xffffffff) 
		DEBUG("PciFind[%d]: dev=%d func=%d pci_data=%lx\n",i,
			pcilist[i].dev, pcilist[i].func, pci_data);
            switch (pci_data) {
                case ID_8111:
                case ID_81801:
                case ID_81801AB:
                case ID_82801BA:
                case ID_82801CA:
                case ID_CSB5:
                case ID_CSB6:
                case ID_OSB4:
                case ID_ICH4:
                case ID_ICH5:
                case ID_ICH6:
		case ID_HanceRapids:
                    found = TRUE;
                    break;
                default:
                    found = FALSE;
                    break;
            }
#ifdef OLD
            if (found == TRUE)
                break;
        }// End of inner For 
#endif
        if (found == TRUE)
            break;
    }// End of outer For
    
    (*ID)	= pci_data;
    (*pdev)	= (UINT)pcilist[i].dev;
    (*pfunc)    = (UINT)pcilist[i].func;

    return found;
}

ULONG GetPciDevId(UCHAR dev, UCHAR func)
{
    ULONG current_pci_id, pci_config_addr;
    
    pci_config_addr  = 0x80000000 | ((dev<<11)&0xFF00);
    pci_config_addr |= ((func<<8)&0x0700);
    
#ifdef TEST
    DEBUG("GetPciDevId(%d,%d) pci_addr=%lx\n",dev,func,pci_config_addr);
#endif
    current_pci_id = PciReadDWord(pci_config_addr);
    
    return current_pci_id;
}


ULONG PciReadDWord (ULONG deviceId)
{

    ULONG addrReg = PCI_CONFIG_ADDRESS;
    ULONG dataReg = PCI_CONFIG_DATA;
    ULONG dataValue;
    
    WritePortUlong(addrReg, deviceId);
    ReadPortUlong(dataReg, &dataValue);
    
    return dataValue;

}

VOID PciWriteDWord(ULONG deviceId, ULONG data)
{
    ULONG addrReg = PCI_CONFIG_ADDRESS;
    ULONG dataReg = PCI_CONFIG_DATA;
    
    WritePortUlong(addrReg, deviceId);
    WritePortUlong(dataReg, data);
}

