/*----------------------------------------------------------
* File Contents:
*     smb.c
* Project: IMB Driver
*-----------------------------------------------------------
* Copyright (c) 2003-2007 Intel Corporation
*---------------------------------------------------------*/
/*M*                     
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
*M*/
/*---------------------------------------------------------*/
/*
 * $Log: smb.c $
 * Revision 1.6  2006/06/07 10:56:38   ARCress
 * SMBCharact(): only probe first time
 *               2003/11/22 assigned to Intel
 * Revision 1.5  2003/04/01 14:57:54Z  marcela
 * add semaphore
 * Revision 1.4  2003/03/23 07:04:44Z  marcela
 * Add AMD 
 * Remove DEBUG
 * Revision 1.3  2003/01/29 10:45:34Z  marcela
 * Add ICH5
 * Revision 1.2  2002/12/19 08:04:04Z  marcela
 * Add (()) to DEBUG expressions
 * Cancel Timer - not needed
 * Revision 1.1  2002/12/11 07:27:32Z  mks
 * Initial revision
 */
/* Enable DBG for extra DEBUG printk's on every SMBus request/response. */
// #define DBG 1

#if defined(WIN32) || defined(WINVER)
    #include <wdm.h>
#endif
#include "imb_os.h"
#include "imb_if.h"
#include "imb_drv.h"
#include "smb.h"
#include "pci.h"

#define SEND_REQ_DELAY   50000    //  50 msec =  50000 microsec
#define READ_RESP_DELAY 100000    // 100 msec = 100000 microsec
#define SMBUS_TIMEOUT   1000

void OsWaitForSemaphore(void * context);
void OsReleaseSemaphore(void * context);
/*
 * Note that the Delay() function below calls udelay() in Linux which 
 * does a CPU spin for 150+ usec per IPMI command.  
 * Some sleep_on function using schedule_timeout() would be better.
 *        - ARCress
 */

//////////////////////////////////////////////////////////////////////////////
// SmbProbe
//////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       SmbProbe
//  Purpose:    Try to find the SMB interface
//  Context:
//  Returns:    TRUE if SMB found,else  FALSE
//  Parameters:
//  Notes:      none
*F*/
static BOOL SmbProbe(void)
{
    UINT DeviceIndex;
    UINT FunctionIndex;
    ULONG ID;
    BOOL ret;
    ret = PciFind(&ID, &DeviceIndex, &FunctionIndex);
    DEBUG("SMBProbe: ret = %d, Func=%d ID = %04lx, Device=%d\n",
		ret,FunctionIndex,ID,DeviceIndex);
    return(ret);
}

/////////////////////////////////////////////////////////////////////////////
// SmsBufferAvail
/////////////////////////////////////////////////////////////////////////////
/*F*
//  Name:       SmsBufferAvail
//  Purpose:    Check to see if the SMS buffer has anything in it
//  Context:
//  Returns:    TRUE ,if SMS_ATN flag set , else FALSE
//  Parameters:
//  Notes:      none
*F*/
static BOOL
SmsBufferAvail( void )
{
    //sms buffer is not supported
    return FALSE;
}

/*******************************************************************************
*  Routine: SMBCharact 
*  Arguments: SMBCharStruct
*  Returns:
*  Description:
*     This routine finds the pci device and its characteristics 
*******************************************************************************/
static void SMBCharact (SMBCharStruct* pSMBChar)
{
    UINT DeviceAdd;
    UINT FuncIndex;
    ULONG ID;
    
    pSMBChar->baseAddr   = 0;
    pSMBChar->Controller = 0;
    
    if (PciFind(&ID, &DeviceAdd, &FuncIndex)==FALSE)
        return;
    
    DeviceAdd = 0x80000000 | ((DeviceAdd<<11)&0xFF00);
    DeviceAdd |= ((FuncIndex<<8)&0x0700);
    
    DEBUG("SMBCharact[%d]: ID = %lx DeviceAdd = %x\n",FuncIndex,ID,DeviceAdd);
    // Read SMBus controller base address
    switch (ID) {
        case ID_81801:
        case ID_81801AA:
        case ID_81801AB:
        case ID_82801BA:
        case ID_82801CA:
        case ID_ICH4:
        case ID_ICH5:
        case ID_ICH6:
        case ID_HanceRapids:
            pSMBChar->Controller = INTEL_SMBC;
            pSMBChar->baseAddr = (WORD)(PciReadDWord(DeviceAdd + 0x20) & 0xFFFE);
            break;
        case ID_OSB4:
        case ID_CSB5:
        case ID_CSB6:
            pSMBChar->Controller = SW_SMBC;
            pSMBChar->baseAddr = (WORD)(PciReadDWord(DeviceAdd + 0x90) & 0xFFFE);
            break;
        case ID_8111:
            // SMBus 2.0 handling
            pSMBChar->Controller = AMD_SMBC;
            pSMBChar->baseAddr = (WORD)(PciReadDWord(DeviceAdd + 0x10) & 0xFFFE);
            PciWriteDWord(DeviceAdd + AMD_SMB_BASE_2_EN, (DWORD)(1));
            break;
        default:
            break;
    } //End of Switch

    // Debug Printings
    DEBUG("SMBCharact Bus ID       = %lx\n", ID);
    DEBUG("SMBCharact Controller   = %d\n", pSMBChar->Controller);
    DEBUG("SMBCharact base address = %x\n", pSMBChar->baseAddr);
/* ********Sample Debug Output************* 
 SMBProbe: ret = 1, Func=3 ID = 25a48086, Device=31
 IMB driver found BMC via SMBus at 0x84
 SMBCharact[3]: ID = 25a48086 DeviceAdd = 8000fb00
 SMBCharact Bus ID       = 25a48086
 SMBCharact Controller   = 1
 SMBCharact base address = 400
 * ********Sample Debug Output*************/
    return;
}

//
static IMB_STATUS
SendBmcRequest (
        BmcRequest *    request,
        DWORD           reqLength,
        BmcResponse *   response,
        DWORD *         respLength,
        void *          context,
        DWORD           ipmiOldFlag
        )
{
    // Find SMBus controller and base address
    static SMBCharStruct SMBchars = {0,0};
    int retries = 5;
    
    if(SMBchars.baseAddr == 0)  /*only probe this the first time*/
        SMBCharact(&SMBchars);
    
    // Check Valid Address
    if(SMBchars.baseAddr == 0)
        return IMB_SEND_ERROR;
    
    // ------------------------------------
    // Send Request - Retry 5 times at most
    // ------------------------------------
    
    OsWaitForSemaphore(context);
    
    do{
        if (SendRequest(request, reqLength, &SMBchars, context) == 0){
            // Success
            break;
        }
    }while (0 < retries--);
    
    if (0 >= retries){
        OsReleaseSemaphore(context);
        return IMB_SEND_ERROR;
    }
    
    // -------------------------------------
    // Read Response - Retry 5 times at most
    // -------------------------------------
    retries = 5;
    
    do{
        // Clean Response Structure
        response->cCode = response->cmd = response->data[0] = response->nfLn = 0;
        
        if ((*respLength = ReadResponse(response, &SMBchars, context))!=(-1)){
            // Success
            break;
        }
    }while (0 < retries--);
    
    if (0 >= retries){
        OsReleaseSemaphore(context);
        return IMB_SEND_ERROR;
    
    }
    
    OsReleaseSemaphore(context);
    return IMB_SUCCESS;    
}

static int
SendRequest(
    BmcRequest *    req,
    int             length,
    SMBCharStruct * smbChar,
    void *          context
    )
{
    BYTE        data;
    // IpmbMsg_t * msgBuf = (IpmbMsg_t *) req;
    int         i, timeout = 0;
    os_timer_t  SendReqTimer;
    BYTE status;
    
    // Delay of 50 ms before request is sent
    Delay(SEND_REQ_DELAY);
    
    // Handle AMD chipset
    if (smbChar->Controller == AMD_SMBC){
        
        // SMBus 2.0 handling 
        if(amd_ec_write(smbChar->baseAddr,(BYTE)AMD_SMB_ADDR,(BYTE)(_smbaddr & 0xFE)))
            return -1;
        if(amd_ec_write(smbChar->baseAddr,(BYTE)AMD_SMB_CMD,(BYTE)0x02))
            return -1;
        if(amd_ec_write(smbChar->baseAddr,(BYTE)AMD_SMB_BCNT,(BYTE)(length)))
            return -1;
        
        for (i = 0; i < length; i++) {
            if(amd_ec_write(smbChar->baseAddr, (BYTE)(AMD_SMB_DATA+i), *(BYTE*)((BYTE*)req + i)))
                return -1;
        } // end of for
        
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_PRTCL, AMD_SMB_BLOCK_WRITE))
            return -1;
        
        do {
            if(amd_ec_read(smbChar->baseAddr, (BYTE)AMD_SMB_STS, &data))
                return -1;
        } while ((~data & AMD_SMB_STS_DONE) && (timeout++<2000));
        
        if((~data & AMD_SMB_STS_DONE) || (data & AMD_SMB_STS_STATUS))
            return -1;
        
        
    // End of AMD Chipset Handling
    } else {
        // Handle Intel & ServerWorks Chipsets
        // Clear all status bits, Host Status Register
        switch (smbChar->Controller) {
            // ServerWorks Status Register
            case SW_SMBC:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR;
                break;
            // Intel Status Register
            case INTEL_SMBC:
	    default:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR|ICH_HST_STA_BYTE_DONE_STS;
                break;
        }// End of Switch
        
        // Status Register
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_STA), status);
        // Block protocol, Host Control Register
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_CNT), (BYTE)(ICH_HST_CNT_SMB_CMD_BLOCK));
        // IPMI command, Host Command Register
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_CMD), (BYTE)(0x02));
        // Slave address, Host Address Register
        WritePortUchar(((smbChar->baseAddr)+ICH_XMIT_SLVA), (BYTE)(_smbaddr));
        // Block length, Host DATA0 Register
        WritePortUchar(((smbChar->baseAddr)+ICH_D0), (BYTE)(length));
        // Initialize timer
#ifdef DBG
	printk("imb SendRequest: write(%x,%x) write(%x,%x) write(%x,%x) write(%x,%x) write(%x,%x)\n",
        (smbChar->baseAddr)+ICH_HST_STA, status,
        (smbChar->baseAddr)+ICH_HST_CNT, ICH_HST_CNT_SMB_CMD_BLOCK,
        (smbChar->baseAddr)+ICH_HST_CMD, 0x02,
        (smbChar->baseAddr)+ICH_XMIT_SLVA, _smbaddr,
        (smbChar->baseAddr)+ICH_D0, length
	);
#endif
        OsInitTimer(&SendReqTimer, SMBUS_TIMEOUT);
        
        switch (smbChar->Controller) {
            // Handle Intel Chipset
            case INTEL_SMBC:
                // the first byte
                WritePortUchar(((smbChar->baseAddr)+ICH_BLOCK_DB), *(BYTE *)((BYTE *)req ));
                // Block protocol and Start, Host Control Register
                WritePortUchar(((smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK);
#ifdef DBG
	printk("imb SendRequest byte1: write(%x,%x) write(%x,%x)\n",
                (smbChar->baseAddr)+ICH_BLOCK_DB, *((BYTE *)req),
                (smbChar->baseAddr)+ICH_HST_CNT, (ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK)
	);
#endif
                // Send byte by byte
                for ( i = 1; i < length; i++ ) {
                    // Start timer
                    OsSetTimer(&SendReqTimer);
                    do {
                        // Read host status register
                        ReadPortUchar((smbChar->baseAddr+ICH_HST_STA), &data);
                        
                        if (OsTimerTimedout(&SendReqTimer))
                            return -1;
                        
                    // Any error or Interrupt bit or byte done bit set ?
                    } while ((data & status) == 0);
                    // End Timer
                    OsCancelTimer(&SendReqTimer);
                    // Check for byte completion in block transfer
                    if ((data & ICH_HST_STA_BYTE_DONE_STS) != ICH_HST_STA_BYTE_DONE_STS)
                         break;
                    // Write next byte
                    WritePortUchar(((smbChar->baseAddr)+ICH_BLOCK_DB), *(BYTE *)((BYTE *)req + i));
                    // Clear status bits, Host Status Register
                    WritePortUchar((smbChar->baseAddr), (UCHAR)(data & status));
#ifdef DBG
	printk("imb SendRequest byteN: write(%x,%x) write(%x,%x)\n",
                (smbChar->baseAddr)+ICH_BLOCK_DB, *((BYTE *)(req + i)),
                (smbChar->baseAddr),(data & status) );
#endif
                } // End of for - Send byte by byte
            break; // End of Intel Chipset Handling
            // Handle ServerWorks Chipset
            case SW_SMBC:
                // Block length, Host DATA1 Register
                WritePortUchar(((smbChar->baseAddr)+ICH_D1), (BYTE)(length));
                // Reset block index
                ReadPortUchar(((smbChar->baseAddr)+ICH_HST_CNT), &data);
                // Put all bytes of the message to Block Data Register
                for ( i = 0; i < length; i++ )
                    WritePortUchar(((smbChar->baseAddr)+ICH_BLOCK_DB), *(BYTE *)((BYTE *)req + i));
                // Block protocol and Start, Host Control Register
                WritePortUchar(((smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK);
                // Start timer
                OsSetTimer(&SendReqTimer);
                do {
                    // Read host status register
                    ReadPortUchar((smbChar->baseAddr+ICH_HST_STA), &data);
                    
                    if (OsTimerTimedout(&SendReqTimer))
                        return -1;
                    
                } while ((data & status) == 0);    // Any error or Interrupt bit set ?
                // End Timer
                OsCancelTimer(&SendReqTimer);
            break; // End of ServerWorks Chipset Handling
        } // End of Switch Controller
        
        // Clear status bits, Host Status Register
        WritePortUchar((smbChar->baseAddr), (UCHAR)(data & status));
#ifdef DBG
	printk("imb SendRequest end: write(%x,%x)\n",
        (smbChar->baseAddr), (data & status));
#endif
        
        if (data & ICH_HST_STA_ALL_ERRS)
            return -1;
         
    } // End of Intel + ServerWorks
    
    // Success
    return 0;
}

static int
ReadResponse(
    BmcResponse *  resp,
    SMBCharStruct *  smbChar,
    void * context
    )
{
    BYTE data, status, dummy, length;
    BYTE *  msgBuf = (BYTE *) resp;
    int     i, timeout =0;
    os_timer_t  SendReqTimer;
    
    // Delay current thread - time to delay in uSecs
    // 100 msec = 100000 microsec
    Delay(READ_RESP_DELAY);
    
    // Handle AMD chipset
    if (smbChar->Controller == AMD_SMBC){
        // SMBus 2.0 handling 
        
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_ADDR, (BYTE)(_smbaddr & 0xFE)))
            return -1;
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_CMD, (BYTE)0x03))
            return -1;
        if(amd_ec_write(smbChar->baseAddr, (BYTE)AMD_SMB_PRTCL, (BYTE)AMD_SMB_BLOCK_READ))
            return -1;
        
        do {
            if(amd_ec_read(smbChar->baseAddr, (BYTE)AMD_SMB_STS, &data))
                return -1;
        }while ((timeout++<2000)&&(~data & AMD_SMB_STS_DONE));
        
        if((~data &AMD_SMB_STS_DONE) || (data & AMD_SMB_STS_STATUS))
            return -1;
        
        if(amd_ec_read(smbChar->baseAddr, (BYTE)AMD_SMB_BCNT, &length))
            return -1;
        
        if (length > 32)    length=32;
        
        for (i = 0; i < length; i++) {
            if(amd_ec_read(smbChar->baseAddr, (BYTE)(AMD_SMB_DATA + i),&msgBuf[i]))
                return -1;
        }
        // DUMP_ATAPI expects 0 in offset 7
        if(amd_ec_write(smbChar->baseAddr, (BYTE)(AMD_SMB_DATA + 3), (BYTE)(0x00)))
            return -1;        
        
    } else {
        // Handle Intel & ServerWorks Chipsets
        switch (smbChar->Controller) {
            case SW_SMBC:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR;
                break;
            case INTEL_SMBC:
	    default:
                status = ICH_HST_STA_ALL_ERRS|ICH_HST_STA_INTR|ICH_HST_STA_BYTE_DONE_STS;
                break;
        }// End of Switch
        
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_STA), status);
        // Block protocol, Host Control Register
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_SMB_CMD_BLOCK);
        // Slave address, Host Address Register
        WritePortUchar(((smbChar->baseAddr)+ICH_XMIT_SLVA), (BYTE)(_smbaddr|ICH_XMIT_SLVA_READ));
        // IPMI command, Host Command Register
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_CMD), 0x03);
        // Reset block index
        ReadPortUchar(((smbChar->baseAddr)+ICH_HST_CNT), &data);
        // Block protocol and Start, Host Control Register
        WritePortUchar(((smbChar->baseAddr)+ICH_HST_CNT), ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK);
        // Initialize timer
#ifdef DBG
	printk("imb ReadResponse: write(%x,%x) write(%x,%x) write(%x,%x) write(%x,%x) read(%x,%x) write(%x,%x)\n",
        (smbChar->baseAddr)+ICH_HST_STA, status,
        (smbChar->baseAddr)+ICH_HST_CNT, ICH_HST_CNT_SMB_CMD_BLOCK,
        (smbChar->baseAddr)+ICH_XMIT_SLVA, (_smbaddr|ICH_XMIT_SLVA_READ),
        (smbChar->baseAddr)+ICH_HST_CMD, 0x03,
        (smbChar->baseAddr)+ICH_HST_CNT, data,
        (smbChar->baseAddr)+ICH_HST_CNT, (ICH_HST_CNT_START | ICH_HST_CNT_SMB_CMD_BLOCK)
	);
#endif
        OsInitTimer(&SendReqTimer, SMBUS_TIMEOUT);
        // Start timer
        OsSetTimer(&SendReqTimer);
        
        do {
            // Read host status register
            ReadPortUchar((smbChar->baseAddr+ICH_HST_STA), &data);
            
            if (OsTimerTimedout(&SendReqTimer))
                return -1;
            
        } while ((data & status) == 0); // Any error or Interrupt bit set ?
        // End Timer
        OsCancelTimer(&SendReqTimer);
        
        // Clear status bits, Host Status Register
        WritePortUchar((smbChar->baseAddr), (BYTE)(data & status));
        
        if (data & ICH_HST_STA_ALL_ERRS)
           return -1;
        
        // Block length, Host DATA0 Register
        ReadPortUchar(((smbChar->baseAddr)+ICH_D0), &length);
        if (length >MAX_ISA_LENGTH)
            length = MAX_ISA_LENGTH;
        switch (smbChar->Controller) {
            case INTEL_SMBC:
                // Read the first byte
                ReadPortUchar((smbChar->baseAddr+ICH_BLOCK_DB), &(msgBuf[0]));
                // Put all bytes of the message to Block Data Register
                for ( i = 1; i < length; i++ ) {
                    if (i == (length-1)) {
                        // Set Last Byte bit
                        ReadPortUchar(((smbChar->baseAddr)+ICH_HST_CNT), &dummy);
                        WritePortUchar(((smbChar->baseAddr)+ICH_HST_CNT), (UCHAR)(dummy | ICH_HST_CNT_LAST_BYTE));
                    }
                    // Clear status bits, Host Status Register
                    WritePortUchar((smbChar->baseAddr), (UCHAR)(data & status));
                    do {
                        // Read host status register
                        ReadPortUchar((smbChar->baseAddr+ICH_HST_STA), &data);
                    } while ((data & status) == 0);    // Any error or Interrupt bit set ?
                    
                    if (data & ICH_HST_STA_ALL_ERRS)
                        return -1;
                    
                    ReadPortUchar((smbChar->baseAddr+ICH_BLOCK_DB), &(msgBuf[i]));
                }
                break;
            case SW_SMBC:
                // Put all bytes of the message to Block Data Register
                for ( i = 0; i < length; i++ ) {
                    ReadPortUchar((smbChar->baseAddr+ICH_BLOCK_DB), &(msgBuf[i]));
                }
                break;
        }// End of Switch
        
        if (data & ICH_HST_STA_ALL_ERRS)
            return -1;
    } // End of Intel + ServerWorks
    
    return length;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
// Amd
/////////////////////////////////////////////////////////////////////////////////////////////////

int amd_ec_wait_write( WORD baseAddress )
{
    int timeout = 1000;
    BYTE data;
    
    ReadPortUchar((baseAddress + AMD_EC_SC), &data);
    
    while(timeout-- && (data & AMD_EC_SC_IBF)){
        Delay(1);
        ReadPortUchar((baseAddress + AMD_EC_SC), &data);
    }
    
    if (!timeout)
        return -1;
    
    return 0;
}

int amd_ec_wait_read( WORD baseAddress )
{
    int timeout = 1000;
    BYTE data;
    
    ReadPortUchar((baseAddress + AMD_EC_SC), &data);
    
    while(timeout-- && (~data & AMD_EC_SC_OBF)){
        Delay(1);
        ReadPortUchar((baseAddress + AMD_EC_SC), &data);
    }
    
    if (!timeout)
        return -1;
    
    return 0;
}

int amd_ec_read(WORD baseAddress, BYTE address, BYTE *data)
{
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar(baseAddress + AMD_EC_CMD , AMD_EC_CMD_RD);
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar(baseAddress + AMD_EC_DATA , address);
    
    if (amd_ec_wait_read(baseAddress)){
        return -1;
    }
    
    ReadPortUchar((baseAddress + AMD_EC_DATA), data);
    
    return 0;
}

int amd_ec_write(WORD baseAddress, BYTE address, BYTE data)
{
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar(baseAddress + AMD_EC_CMD , (BYTE)AMD_EC_CMD_WR);
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar(baseAddress + AMD_EC_DATA, address);
    
    if (amd_ec_wait_write(baseAddress)){
        return -1;
    }
    
    WritePortUchar(baseAddress + AMD_EC_DATA, data);
    
    return 0;
}

//
// This structure is the only symbol exported by this module
// It provides the interface to discover and access the KCS if
//
ImbInterface SmbImbIf = {

    SendBmcRequest,    // sends BMC request and return response
    SmsBufferAvail,    // indicates whether the SMS buffer is available
    SmbProbe           // indicates whether there is a SMB IF on this platform

};

