/*----------------------------------------------------------
* File Contents:
*     smb.h
* Project: IMB Driver 
*-----------------------------------------------------------
* Copyright (c) 2003-2007 Intel Corporation
*---------------------------------------------------------*/
/*M*                     
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
*M*/
/*---------------------------------------------------------*/
/*
 * $Log: smb.h $
 * Revision 1.2  2003/03/23 04:50:40Z  marcela
 * Add amd functions
 * Revision 1.1  2002/12/11 07:24:38Z  mks
 * Initial revision
 */

#define SMB_DATA_ADDRESS = 0x000

//
// Default SMS and SMM interface addresses
//
#define ISA_SMS_DATA          0xCA2
#define ISA_SMS_CMD_STATUS    0xCA3

#define ISA_SMM_DATA          0xCA4
#define ISA_SMM_CMD_STATUS    0xCA5
//
// State bits based on S1 & S0 below
//
#define ISA_STATE_MASK        0xC0
#define ISA_IDLE_STATE        0x00
#define ISA_READ_STATE        0x40
#define ISA_WRITE_STATE       0x80
#define ISA_ERROR_STATE       0xC0
//
// Status Register Bits
//
#define ISA_S1_FLAG           0x80
#define ISA_S0_FLAG           0x40
// RESERVED                   0x20
// RESERVED                   0x10
#define ISA_CD_FLAG           0x08
#define ISA_SMS_MSG_FLAG      0x04
#define ISA_IBF_FLAG          0x02
#define ISA_OBF_FLAG          0x01
//
// ISA interface register access defines
//
#define ISA_OBF(a)            ((BYTE)(_inp(a+1) & ISA_OBF_FLAG))
#define ISA_IBF(a)            ((BYTE)(_inp(a+1) & ISA_IBF_FLAG))
#define ISA_MSG(a)            ((BYTE)(_inp(a+1) & ISA_SMS_MSG_FLAG))
#define ISA_STATE(a)          ((BYTE)(_inp(a+1) & ISA_STATE_MASK))
#define ISA_STATUS(a)         ((BYTE)(_inp(a+1)))
#define ISA_READ_BMC_DATA(a)  ((BYTE)(_inp(a)))
#define ISA_WRITE_BMC_DATA(a,x)    (_outp(a,(unsigned)x))
#define ISA_WRITE_BMC_CMD(a,x)     (_outp(a+1,(unsigned)x))
//
// SMS Transfer Stream Control Codes
//                                
#define ISA_GET_STATUS_ABORT   0x60
#define ISA_WRITE_START        0x61
#define ISA_WRITE_END          0X62
#define ISA_READ               0X68

#define BMC_SLAVE_ADDR         0x20
#define MAX_ISA_LENGTH         35

#define ISA_SMS_TIMEOUT        5000    //    in miliseconds - 1 second
#define ISA_SMM_TIMEOUT        100     //    in microseconds
//
// mBMC Address
//
extern int _smbaddr;

static BOOL
SmsBufferAvail( void );

static IMB_STATUS
SendBmcRequest (
        BmcRequest *    request,
        DWORD           reqLength,
        BmcResponse *   response,
        DWORD *         respLength,
        void *          context,
        DWORD           ipmiOldFlag
        );

static int 
ReadResponse(
        BmcResponse *   resp, 
        SMBCharStruct * smbChar,
        void *          context
        );  
static int 
SendRequest(
        BmcRequest *    req, 
        int             length, 
        SMBCharStruct * smbChar, 
        void *          context
        );
//
// amd read and write functions
//
int amd_ec_wait_write( WORD baseAddress );
int amd_ec_wait_read( WORD baseAddress );
int amd_ec_read(WORD baseAddress, BYTE address, BYTE *data);
int amd_ec_write(WORD baseAddress, BYTE address, BYTE data);



