/*M*
 *  $Workfile:   getboard.c  $
 *  $Revision:   1.1  $
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
 *M*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/param.h>
#include <sys/mman.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
 #define PARAM_TABLE_SIGN						"PTBL\0\0\0\0"
#define PARAM_TABLE_SIGN_SIZE					8
#define OFFSET_TO_BIOS_VERSION_STRING			12
#define STARTADDRESS							0xE0000
#define ADDRESSLENGTH							0x1FFF0
#define FIR_BOARD_ID            		"CD"
#define BUCKEYE_BOARD_ID        		"DA"
/*
 * Note the following board names are not used anymore in this program,
 * but I have left them here for reference. Supply the board names as
 * command line arguments to getboard and those names will be compared
 * to what is found in the bios. MR
 *
#define MADRONA_BOARD_ID        		"MADRO"
#define REDWOOD_BOARD_ID        		"REDWD"
#define NITSHADE_ID				"NIGHT"
#define NITELITE_ID            			"NITELT0"
#define SITKA_BOARD_ID				"S450NX"
#define BEAR_BOARD_ID				"ASPN0"
#define POLAR_BOARD_ID                          "ASPN1"
#define LANCEWOOD_ID 				"L440GX"
#define SABER_BOARD_ID 				"SABR0"
#define KOA_BOARD_ID 				"8K4W0"
#define HEMLOC_BOARD_ID 			"AH8L020"
#define HEMLOC2_BOARD_ID 			"8H4W0"
#define PINE_BOARD_ID 				"2P08PD"
#define KOA_RCC_BOARD_ID			"SKA40"
*/
#define IMBDRIVER "/dev/mem"
/* The following is needed for Linux */
#define PAGESIZE EXEC_PAGESIZE
typedef union
{
    unsigned long     ulSegmentOffset;
    unsigned short    ausAddress[2];
}AddressConversionUnion;
unsigned long *
convertAddrToLinearAddress (
    unsigned long ulSegmentOffset);
char pname[30]="";
char * GetBaseboardType();
int main(int argc, char *argv[])
{
	if (argc < 2)  /*atleast one board type as argument expected */
	{
		printf("%s <one or more board types>\n", argv[0]);
		exit(1);
	}
	printf("%s\n", GetBaseboardType(argc,argv));
	exit(0);
}
char * GetBaseboardType(int argc, char **argv)
{
	caddr_t	startpAddress = (caddr_t) STARTADDRESS,	
			startvAddress;
	char	biosVersionStr[64];
	char	*bios_ver_str;
	int 	length = ADDRESSLENGTH, fd, i,  j, diff;
	if(getuid()!= 0)
	{
		fprintf(stderr, "Sorry, Only root can execute this program\n");
		exit(1);
	}
	fd = open(IMBDRIVER, O_RDONLY);
	if(fd < 0 )
	{
		fprintf(stderr, "Could not open %s\n", IMBDRIVER);
		exit(1);
	}
	/* aliging the offset to a page boundary and adjusting the length */
	diff = (unsigned long)startpAddress %(unsigned long) PAGESIZE;
	startpAddress -= diff;
	length += diff;
	startvAddress = (caddr_t)mmap((caddr_t)0, length, PROT_READ, MAP_SHARED,
				fd, (off_t) startpAddress);
	if(startvAddress == (caddr_t) -1)
	{
		perror("mmap failed \n");
		return pname;
	}
	for(i=0; i < length; i++)
	{
		if (!memcmp(startvAddress + i, PARAM_TABLE_SIGN, 
							PARAM_TABLE_SIGN_SIZE))
			break;
		
		for (j = 1; j < argc; j++)
		{
			if (!memcmp(startvAddress + i, argv[j], strlen(argv[j])) )
			{
				sprintf (pname, argv[j]);
				return pname;
			}
		}
	}
	if (i == length)
	{
		sprintf(pname, "unknown");
		return pname;
	}
	bios_ver_str = (caddr_t)convertAddrToLinearAddress(*((long *)(startvAddress
						+ i + OFFSET_TO_BIOS_VERSION_STRING)));
	/* aliging the offset to a page boundary and adjusting the length */
	diff = (unsigned long)bios_ver_str % (unsigned long)PAGESIZE;
	bios_ver_str -= diff;
	munmap((caddr_t)startpAddress, length);
	startvAddress = (caddr_t) mmap((caddr_t) 0, PAGESIZE, 
					PROT_READ, MAP_SHARED, fd, (off_t) bios_ver_str);
	if(startvAddress == (caddr_t) -1)
	{
		perror("mmap failed \n");
		return pname;
	}
	memcpy(biosVersionStr, startvAddress + diff, 26);
	biosVersionStr[26] = 0;
	if (!memcmp(&biosVersionStr[21], FIR_BOARD_ID, 2)) 
		sprintf(pname, "fir");
	else if (!memcmp(&biosVersionStr[21], BUCKEYE_BOARD_ID, 2)) 
		sprintf(pname, "buckeye");
	else	
	{
		sprintf(pname, "unknown");
		close(fd);
	}
	close(fd);
	return pname;
}
unsigned long *
convertAddrToLinearAddress (
	unsigned long ulSegmentOffset)
{
    AddressConversionUnion  addressConversionUnion;
    addressConversionUnion.ulSegmentOffset = ulSegmentOffset;
	return ( (unsigned long *) 
			( ( (unsigned long) addressConversionUnion.ausAddress[1] << 4)
             + (unsigned long) addressConversionUnion.ausAddress[0]) );
}
