
#ifndef _HARDWARE_DEFS_
#define _HARDWARE_DEFS_

#define DEFAULT_I2C_ADDR    0x02
#define OWN_ADDRESS			  0xa0		// Baseboard 8584 controller 

#define OWN_ID                (OWN_ADDRESS >> 1)        // OWN I2C address, my address 0xa0

//
// I2C 8584 status bit definitions, used for check 8584 status
//

#define BB    0x01             // ~bus busy             0 = busy     1 = free
#define LAB   0x02             // lost arbitration      0 = false    1 = true
#define AAS   0x04             // addrsed as slave      0 = false    1 = true
#define LRB   0x08            // last received bit     0 = acknwl   1 = not ack
#define AD0   0x08            // recv genrl call addr  0 = false    1 = true
#define BER      0x10            // bus error             0 = false    1 = true
#define STS   0x20            // external stop recv    0 = false    1 = true
#define INIT  0x40            // ~initialized 8584     0 = true     1 = false
#define PIN   0x80               // pending interrupt not    0 = servce   1 = normal

//
// I2C 8584 control bit definitions
//
#define ACK 0x01             // automatic acknowlege 
#define STO    0x02            // stop serial bus control
#define STA    0x04            // star serial bus control
#define ENI    0x08            // enable interrupt    
#define ES2    0x10            // control selection
#define ES1    0x20            // control selection
#define ES0    0x40            // enable serial out
#define PIN 0x80            // pending interrupt not


//
// I2C 8584 control registers index definitions
//
#define REPEAT_START    ES0+STA+ACK
#define NACK            ES0
#define STOP            PIN+ES0+STO+ACK
#define START            ES0+STA+ACK+PIN
#define CLOCK             PIN+ES1
#define OWNADDRESS      PIN
#define IDLE_MODE        PIN+ES0+ACK            


//
// constants to use for selecting the PCF8584 clock rate 
//
#define CLK_3MHZ        0x00        // 3 mhz
#define CLK_4430KHZ    0x10        // 4.43 mhz
#define CLK_6MHZ          0x14        // 6 mhz
#define CLK_8MHZ        0x18        // 8 mhz
#define CLK_12MHZ        0x1C        //12 mhz

//
// constants to use for selecting the I2C 8584 slave clock rate 
//
#define SCL_90KHZ        0x00       // 90 khz
#define SCL_45KHZ        0x01    // 45 khz
#define SCL_11KHZ           0x02    // 11 khz
#define SCL_1500HZ      0x03    // 1.5 khz
// muoe962373 - jtt - 7/26/96
// Clock frequency is 8 MHz
#define I2C_CLOCK        CLK_8MHZ+SCL_90KHZ        // Freq=8Mhz, SCL=90Khz

//
// with a 8.33mhz freq it takes approx 8 clocks or 1usecs when reading this port
//

#define    DMA_PAGE_REG        0x80    // rsvp dma page reg to read to kill time
                                    // from hp's code ???TLD






#define I2C_PORT_DELAY		20




#endif // _HARDWARE_DEFS_



