
/*M*
//
//  Purpose:	This file contains the defines for the I2C resources for 4300
//	class systems.  
//
//  Contents:
//		Defines for the I2C addresses of cabinet resources
//		Defines for data returned from cabinet I2C devices
//
*M*/

#ifndef	_I2CMAP_H
#define	_I2CMAP_H

/*D*
//
// Internal I2C Address map
//
*D*/

#define	I2C_BASEIO		0x40
#define	I2C_MEMIO		0x42
#define	I2C_POWER		0x44
#define	I2C_POWER_STATUS	0x48
#define	I2C_LARGE_MEMIO		0x4A
#define	I2C_FAN_REG1		0x4C
#define	I2C_FAN_REG2		0x4E
#define	I2C_SMB_SLAVE		0x52

#define	I2C_CPUIO1		0x70
#define	I2C_CPUIO2		0x72
#define I2C_CABINET_ID          0x78

#define	I2C_CPUAD1		0x90
#define	I2C_CPUAD2		0x92
#define	I2C_SCSI1		0x9C
#define	I2C_SCSI2		0x9E
#define	I2C_BASEBOARD		0xA0
#define	I2C_FRONT_PANEL		0xA2
#define	I2C_PS1EEPROM		0xAC
#define	I2C_PS2EEPROM		0xA6
#define	I2C_PS3EEPROM		0xA4
#define	I2C_MEMEEPROM		0xAE

#pragma pack(1)
/*D
//
//  Structure returned from Power Supply EEPROM
//
*D*/
#define	PWR_PART_NUMBER_SIZE	10
#define PWR_NUMBER_OF_VOLTAGES	16

typedef struct {
	unsigned char	programCode;
	unsigned char	dataFormatCode;
	unsigned short	supplyPowerOutput;
	unsigned char	partNumber[PWR_PART_NUMBER_SIZE];
	unsigned char	revisionLevel;
	unsigned char	manufactureDay;
	unsigned char	manufactureMonth;
	unsigned short	manufactureYear;
	unsigned char	reserved1[12];
	unsigned char	checksum;
	struct {
		short	supplyVoltage;
		short	supplyVoltageCurrent;
	} supplyVoltages[PWR_NUMBER_OF_VOLTAGES];
} POWER_SUPPLY_EEPROM;

/*D
//
//  Defines for Power supply present/fault register
//
*D*/
	
#define	POWER_SUPPLY1_ABSENT	0x01	// 0 = power supply present
#define	POWER_SUPPLY1_FAIL	0x02
#define	POWER_SUPPLY2_ABSENT	0x04
#define	POWER_SUPPLY2_FAIL	0x08
#define	POWER_SUPPLY3_ABSENT	0x10
#define	POWER_SUPPLY3_FAIL	0x20
#define POWER_UNUSED_BITS       0xc0   // reserved bits

/*D
//
//  Defines for Power supply Status register
//
*D*/

#define	AC_LINE_OK		0x01	// 0 = No AC Line voltage
#define	UPS_BATTERY_ABSENT	0x02	// 0 = Internal UPS battery is present
#define	UPS_BATTERY_OK		0x04	// 0 = Internal UPS battery is low
#define	POWER_SUPPLY_FAN_FAIL	0x08	// One or more power supply fans failed
#define	POWER_GOOD		0x10	// System voltages within tolerances
#define	POWER_ON_OFF		0x20	// Power on/off to supplies (0=on,1=off)
#define SECOND_BASEBOARD_ABSENT 0x40    // 0 = secondary baseboard present
#define PF_SHUT                 0x80    // power shutdown control bit (r/w)

/*D
//
//  Defines for fan fail status register
//
*D*/
#define	FAN1_FAIL		0x01
#define	FAN2_FAIL		0x02
#define	FAN3_FAIL		0x04
#define	FAN4_FAIL		0x08
#define	FAN5_FAIL		0x10
#define	FAN6_FAIL		0x20
#define	FAN7_FAIL		0x40
#define	FAN8_FAIL		0x80

/*D
//
//  Defines for fan fail/board ID status register
//
*D*/
#define	FAN9_FAIL		0x01
#define	FAN10_FAIL		0x02
#define	FAN11_FAIL		0x04
#define	FAN12_FAIL		0x08
#define BATTERY_CHARGER_OK      0x10       // 0 = battery charger fail
#define	POWER_BACKPLANE_ID	0xe0
#define	POWER_BACKPLANE_SHIFT	5

/*D
//
//  Defines for three bit backplane ID
//
*D*/
#define TECTROL_BACKPLANE       0
#define DEFAULT_BACKPLANE_ID    TECTROL_BACKPLANE

/*D
//
//  Defines for Baseboard Module status register(0x40) 
//
*D*/

#define	FANFAILSEL		0x80
#define	PROCESSOR_SELECT	0x7f
#define	PS_FAN_SELECT		0xff
#define	Processor3Failed	0x40
#define	Processor2Failed	0x20
#define	Processor1Failed	0x10
#define	Processor0Failed	0x08
#define	PSFan3Failed		0x40
#define	PSFan2Failed		0x20
#define	PSFan1Failed		0x10
#define	PSFan0Failed		0x08
#define	BASEBOARDID		0x07
#define	BaseboardFab1		0x01
#define	BaseboardFab2		0x02
#define	BaseboardFab3		0x03

/*D
//
//  Defines for Processor Module status register 1
//
*D*/

#define	BRDREVSEL		0x80
#define	PROCESSOR_ERROR_SELECT	0x7f
#define	CPU_REV_SELECT		0xff
#define P6IDMAP		7
#define	P6ID150		0
#define	P6ID180		1
#define	P6ID210		2
#define	P6ID240		3
#define	P6ID166		4
#define	P6ID200		5
#define	P6ID233		6
#define	P6ID266		7
#define P6THERMTRIP1	0x40
#define P6THERMTRIP2	0x20
#define P6IERROR1	0x10
#define P6IERROR2	0x08
#define P6REVMASK	0x38
#define P6REVFab3	0x38

/*D
//
//  Defines for Memory Module status register 1 (all memory modules)
//
*D*/
#define	PCB_ID_BITS		0x03
#define	PCB_1GB_FAB12		0x00
#define	PCB_1GB_FAB3		0x01
#define	PCB_3GB_MODULE		0x03
#define	PCB_1GB_FAB12		0x00

#define	BANK0_SPEED_BITS	0x0c
#define	BANK0_SPEED_SHIFT	2

#define	BANK1_SPEED_BITS	0x30
#define	BANK1_SPEED_SHIFT	4

#define	BANK_SPEED_50NS		0x00
#define	BANK_SPEED_80NS		0x01
#define	BANK_SPEED_70NS		0x02
#define	BANK_SPEED_60NS		0x03

#define	ENABLE_PREMUTE		0x7f	// Enable RAS/CAS multiplexing
#define	DISABLE_PREMUTE		0xff	// Disable RAS/CAS multiplexing
/*D
//
//  Defines for Memory Module status register 2 (4GB memory module)
//
*D*/

#define	BANK2_SPEED_BITS	0x0c
#define	BANK2_SPEED_SHIFT	2

#define	BANK3_SPEED_BITS	0x30
#define	BANK3_SPEED_SHIFT	4
#pragma pack()
#endif	/* I2CMAP_H */
