
#ifndef	_NTDDK_H
#define	_NTDDK_H

#ifndef NULL
#define NULL 0
#endif

#ifndef _WINDOWS_H
#include	"windows.h"
#endif
#include "winioctl.h"

typedef UINT32 NTSTATUS;
#define	NT_SUCCESS(status)		((NTSTATUS)(status) >= 0)

#define STATUS_SUCCESS                   ((NTSTATUS)0x00000000L)
#define STATUS_ALERTED                   ((NTSTATUS)0x00000101L)
#define STATUS_TIMEOUT                   ((NTSTATUS)0x00000102L)
#define STATUS_UNSUCCESSFUL              ((NTSTATUS)0xC0000001L)
#define STATUS_DEVICE_BUSY               ((NTSTATUS)0x80000011L)
#define STATUS_PENDING                   ((NTSTATUS)0x00000103L)
#define STATUS_INVALID_PARAMETER         ((NTSTATUS)0xC000000DL)
#define STATUS_INVALID_DEVICE_REQUEST    ((NTSTATUS)0xC0000010L)
#define STATUS_MORE_PROCESSING_REQUIRED  ((NTSTATUS)0xC0000016L)
#define STATUS_BUFFER_TOO_SMALL          ((NTSTATUS)0xC0000023L)
#define STATUS_FILE_CLOSED               ((NTSTATUS)0xC0000128L)
#define STATUS_OPEN_FAILED               ((NTSTATUS)0xC0000136L)
#define STATUS_INSUFFICIENT_RESOURCES    ((NTSTATUS)0xC000009AL)
#define STATUS_NO_DATA_DETECTED          ((NTSTATUS)0x80000022L)
#define STATUS_NO_SUCH_DEVICE            ((NTSTATUS)0xC000000EL)
#define STATUS_ALLOTTED_EXCEEDED         ((NTSTATUS)0xC000000FL)
#define STATUS_INVALID_DEVICE_STATE      ((NTSTATUS)0xC0000184L)
#define STATUS_IO_DEVICE_ERROR           ((NTSTATUS)0xC0000185L)
#define STATUS_TOO_MANY_OPEN_FILES       ((NTSTATUS)0xC000011FL)
#define STATUS_ACCESS_DENIED             ((NTSTATUS)0xC0000022L)
#define STATUS_BUFFER_OVERFLOW           ((NTSTATUS)0x80000005L)
#define STATUS_CANCELLED                 ((NTSTATUS)0xC0000120L)


/*
 Predefined Value Types.
*/

#define REG_NONE                    ( 0 )   /* No value type */
#define REG_SZ                      ( 1 )   /* Unicode nul terminated string */
#define REG_EXPAND_SZ               ( 2 )   /* Unicode nul terminated string */
                                            /* (with environment variable references) */
#define REG_BINARY                  ( 3 )   /* Free form binary */
#define REG_DWORD                   ( 4 )   /* 32-bit number */
#define REG_DWORD_LITTLE_ENDIAN     ( 4 )   /* 32-bit number (same as REG_DWORD) */
#define REG_DWORD_BIG_ENDIAN        ( 5 )   /* 32-bit number */
#define REG_LINK                    ( 6 )   /* Symbolic Link (unicode) */
#define REG_MULTI_SZ                ( 7 )   /* Multiple Unicode strings */
#define REG_RESOURCE_LIST           ( 8 )   /* Resource list in the resource map */
#define REG_FULL_RESOURCE_DESCRIPTOR ( 9 )  /* Resource list in the hardware description */
#define REG_RESOURCE_REQUIREMENTS_LIST ( 10 )

#ifndef LINUX
/*****Don't need the rest of this file. maybe we can remove it if this is
 * a Linux specific file. Mahendra.
 */

/*
The following values for the RelativeTo parameter determine what the
 Path parameter to RtlQueryRegistryValues is relative to.
*/

#define RTL_REGISTRY_ABSOLUTE     0   /* Path is a full path */
#define RTL_REGISTRY_SERVICES     1   /* \Registry\Machine\System\CurrentControlSet\Services */
#define RTL_REGISTRY_CONTROL      2   /* \Registry\Machine\System\CurrentControlSet\Control */
#define RTL_REGISTRY_WINDOWS_NT   3   /* \Registry\Machine\Software\Microsoft\Windows NT\CurrentVersion */
#define RTL_REGISTRY_DEVICEMAP    4   /* \Registry\Machine\Hardware\DeviceMap */
#define RTL_REGISTRY_USER         5   /* \Registry\User\CurrentUser */
#define RTL_REGISTRY_MAXIMUM      6
#define RTL_REGISTRY_HANDLE       0x40000000    /* Low order bits are registry handle */
#define RTL_REGISTRY_OPTIONAL     0x80000000    /* Indicates the key node is optional */

typedef LONG KPRIORITY;

typedef int KIRQL;
typedef KIRQL	*PKIRQL;

typedef lock_t *KSPIN_LOCK;
typedef lock_t **PKSPIN_LOCK;

/*
 Timer object
*/

typedef struct _KTIMER {
    ULARGE_INTEGER DueTime;
    toid_t TimerListEntry;
    struct _KDPC *Dpc;
    BOOLEAN Inserted;
} KTIMER;

typedef KTIMER *PKTIMER;

typedef
VOID
(*PKDEFERRED_ROUTINE) (
    IN struct _KDPC *Dpc,
    IN PVOID DeferredContext,
    IN PVOID SystemArgument1,
    IN PVOID SystemArgument2
    );

/*
 Deferred Procedure Call (DPC) object
*/

typedef struct _KDPC {
    CSHORT Type;
    CSHORT Size;
    PKDEFERRED_ROUTINE DeferredRoutine;
    PVOID DeferredContext;
    PVOID SystemArgument1;
    PVOID SystemArgument2;
    PULONG Lock;
} KDPC, *PKDPC;

/*
 Event type
*/

typedef enum _EVENT_TYPE {
    NotificationEvent,
    SynchronizationEvent
    } EVENT_TYPE;

/*
  Doubly linked list structure.  Can be used as either a list head, or
  as link words.
*/

typedef struct _LIST_ENTRY {
   struct _LIST_ENTRY *Flink;
   struct _LIST_ENTRY *Blink;
} LIST_ENTRY;
typedef LIST_ENTRY *PLIST_ENTRY;

/*
 Processor modes.
*/

typedef CCHAR KPROCESSOR_MODE;

typedef enum _MODE {
    KernelMode,
    UserMode,
    MaximumMode
} MODE;

/*
 Wait reasons
*/

typedef enum _KWAIT_REASON {
    Executive,
    FreePage,
    PageIn,
    PoolAllocation,
    DelayExecution,
    Suspended,
    UserRequest,
    WrExecutive,
    WrFreePage,
    WrPageIn,
    WrPoolAllocation,
    WrDelayExecution,
    WrSuspended,
    WrUserRequest,
    WrEventPair,
    WrQueue,
    WrLpcReceive,
    WrLpcReply,
    WrVirtualMemory,
    WrPageOut,
    Spare1,
    Spare2,
    Spare3,
    Spare4,
    Spare5,
    Spare6,
    WrKernel,
    MaximumWaitReason
    } KWAIT_REASON;

/*
 Common dispatcher object header
*/

typedef struct _DISPATCHER_HEADER {
    CSHORT Type;
    CSHORT Size;
    LONG SignalState;
    LIST_ENTRY WaitListHead;
} DISPATCHER_HEADER;

/*
 Event object
*/

typedef struct _KEVENT {
    DISPATCHER_HEADER Header;
} KEVENT, *PKEVENT;

/*
 Define the major function codes for IRPs.  The lower 128 codes, from 0x00 to
 0x7f are reserved to Microsoft.  The upper 128 codes, from 0x80 to 0xff, are
 reserved to customers of Microsoft.
*/

#define IRP_MJ_CREATE                   0x00
#define IRP_MJ_CREATE_NAMED_PIPE        0x01
#define IRP_MJ_CLOSE                    0x02
#define IRP_MJ_READ                     0x03
#define IRP_MJ_WRITE                    0x04
#define IRP_MJ_QUERY_INFORMATION        0x05
#define IRP_MJ_SET_INFORMATION          0x06
#define IRP_MJ_QUERY_EA                 0x07
#define IRP_MJ_SET_EA                   0x08
#define IRP_MJ_FLUSH_BUFFERS            0x09
#define IRP_MJ_QUERY_VOLUME_INFORMATION 0x0a
#define IRP_MJ_SET_VOLUME_INFORMATION   0x0b
#define IRP_MJ_DIRECTORY_CONTROL        0x0c
#define IRP_MJ_FILE_SYSTEM_CONTROL      0x0d
#define IRP_MJ_DEVICE_CONTROL           0x0e
#define IRP_MJ_INTERNAL_DEVICE_CONTROL  0x0f
#define IRP_MJ_SHUTDOWN                 0x10
#define IRP_MJ_LOCK_CONTROL             0x11
#define IRP_MJ_CLEANUP                  0x12
#define IRP_MJ_CREATE_MAILSLOT          0x13
#define IRP_MJ_QUERY_SECURITY           0x14
#define IRP_MJ_SET_SECURITY             0x15
#define IRP_MJ_MAXIMUM_FUNCTION         0x15

/*
  Time conversion routines
*/

typedef struct _TIME_FIELDS {
    CSHORT Year;        /* range [1601...]*/
    CSHORT Month;       /* range [1..12]*/
    CSHORT Day;         /* range [1..31]*/
    CSHORT Hour;        /* range [0..23]*/
    CSHORT Minute;      /* range [0..59]*/
    CSHORT Second;      /* range [0..59]*/
    CSHORT Milliseconds;/* range [0..999]*/
    CSHORT Weekday;     /* range [0..6] == [Sunday..Saturday]*/
} TIME_FIELDS;
typedef TIME_FIELDS *PTIME_FIELDS;

  

#define LiFromUlong(a) (RtlConvertUlongToLargeInteger((a)))


/*
 * NT Data types
 */
/*
 Define types of bus information.
*/

typedef enum _BUS_DATA_TYPE {
    Cmos,
    EisaConfiguration,
    Pos,
    CbusConfiguration,
    PCIConfiguration,
    VMEConfiguration,
    NuBusConfiguration,
    PCMCIAConfiguration,
    MPIConfiguration,
    MPSAConfiguration,
    MaximumBusDataType
} BUS_DATA_TYPE, *PBUS_DATA_TYPE;

/*
 Define the I/O bus interface types.
*/

typedef enum _INTERFACE_TYPE {
    Internal,
    Isa,
    Eisa,
    MicroChannel,
    TurboChannel,
    PCIBus,
    VMEBus,
    NuBus,
    PCMCIABus,
    CBus,
    MPIBus,
    MPSABus,
    MaximumInterfaceType
}INTERFACE_TYPE, *PINTERFACE_TYPE;

/*
 Define driver timer routine type.
*/

struct _DEVICE_OBJECT;
struct _IRP;
struct _DRIVER_OBJECT;

typedef
VOID
(*PIO_TIMER_ROUTINE) (
    IN struct _DEVICE_OBJECT *DeviceObject,
    IN PVOID Context
    );

typedef struct _IO_TIMER {
	PIO_TIMER_ROUTINE  TimerRoutine;
	PVOID  Context;
	toid_t	tid;
} IO_TIMER;
typedef IO_TIMER *PIO_TIMER;

/*
 Define driver initialization routine type.
*/

typedef
NTSTATUS
(*PDRIVER_INITIALIZE) (
    IN struct _DRIVER_OBJECT *DriverObject,
    IN PUNICODE_STRING RegistryPath
    );

/*
 Define driver dispatch routine type.
*/

typedef
NTSTATUS
(*PDRIVER_DISPATCH) (
    IN struct _DEVICE_OBJECT *DeviceObject,
    IN struct _IRP *Irp
    );

/*
 Define driver start I/O routine type.
*/

typedef
VOID
(*PDRIVER_STARTIO) (
    IN struct _DEVICE_OBJECT *DeviceObject,
    IN struct _IRP *Irp
    );

/*
 Define driver unload routine type.
*/

typedef
VOID
(*PDRIVER_UNLOAD) (
    IN struct _DRIVER_OBJECT *DriverObject
    );

/*
 Define Device Object (DO) flags
*/

#define DO_UNLOAD_PENDING               0x00000001
#define DO_VERIFY_VOLUME                0x00000002
#define DO_BUFFERED_IO                  0x00000004
#define DO_EXCLUSIVE                    0x00000008
#define DO_DIRECT_IO                    0x00000010
#define DO_MAP_IO_BUFFER                0x00000020
#define DO_DEVICE_HAS_NAME              0x00000040
#define DO_DEVICE_INITIALIZING          0x00000080
#define DO_SYSTEM_BOOT_PARTITION        0x00000100
#define DO_LONG_TERM_REQUESTS           0x00000200
#define DO_NEVER_LAST_DEVICE            0x00000400
#define DO_SHUTDOWN_REGISTERED          0x00000800

/*
 Device Object structure definition
*/

typedef struct _DEVICE_OBJECT {
    CSHORT Type;
    USHORT Size;
    LONG ReferenceCount;
    struct _DRIVER_OBJECT *DriverObject;
    struct _DEVICE_OBJECT *NextDevice;
    struct _DEVICE_OBJECT *AttachedDevice;
    struct _IRP *CurrentIrp;
    PIO_TIMER Timer;
    ULONG Flags;                                /* See above:  DO_...*/
    ULONG Characteristics;                      /* See ntioapi:  FILE_...*/
/*    PVPB Vpb;*/
    PVOID DeviceExtension;
    DEVICE_TYPE DeviceType;
    CCHAR StackSize;
/*    union {*/
/*        LIST_ENTRY ListEntry;*/
/*        WAIT_CONTEXT_BLOCK Wcb;*/
/*    } Queue;*/
    ULONG AlignmentRequirement;
/*    KDEVICE_QUEUE DeviceQueue;*/
    KDPC Dpc;

    /*
      The following field is for exclusive use by the filesystem to keep
      track of the number of Fsp threads currently using the device
    */

    ULONG ActiveThreadCount;
/*    PSECURITY_DESCRIPTOR SecurityDescriptor;*/
/*    KEVENT DeviceLock;*/

    USHORT SectorSize;
    USHORT Spare1;
    LARGE_INTEGER Spare2;

} DEVICE_OBJECT;
typedef struct _DEVICE_OBJECT *PDEVICE_OBJECT; /* ntndi*/

typedef struct _DRIVER_OBJECT {
    CSHORT Type;
    CSHORT Size;

    /*
     The following links all of the devices created by a single driver
     together on a list, and the Flags word provides an extensible flag
     location for driver objects.
    */

    PDEVICE_OBJECT DeviceObject;
    ULONG Flags;

    /*
     The following section describes where the driver is loaded.  The count
     field is used to count the number of times the driver has had its
     registered reinitialization routine invoked.
    */

    PVOID DriverStart;
    ULONG DriverSize;
    PVOID DriverSection;
    ULONG Count;

    /*
     The driver name field is used by the error log thread
     determine the name of the driver that an I/O request is/was bound.
    */

    UNICODE_STRING DriverName;

    /*
     The following section is for registry support.  Thise is a pointer
     to the path to the hardware information in the registry
    */

    PUNICODE_STRING HardwareDatabase;

    /*
     The following section contains the optional pointer to an array of
     alternate entry points to a driver for "fast I/O" support.  Fast I/O
     is performed by invoking the driver routine directly with separate
     parameters, rather than using the standard IRP call mechanism.  Note
     that these functions may only be used for synchronous I/O, and when
     the file is cached.
    */

/*    PFAST_IO_DISPATCH FastIoDispatch;*/

    /*
     The following section describes the entry points to this particular
     driver.  Note that the major function dispatch table must be the last
     field in the object so that it remains extensible.
    */

    PDRIVER_INITIALIZE DriverInit;
    PDRIVER_STARTIO DriverStartIo;
    PDRIVER_UNLOAD DriverUnload;
    PDRIVER_DISPATCH MajorFunction[IRP_MJ_MAXIMUM_FUNCTION + 1];
} DRIVER_OBJECT;
typedef struct _DRIVER_OBJECT *PDRIVER_OBJECT;

/*
 Define the base asynchronous I/O argument types
*/

typedef struct _IO_STATUS_BLOCK {
    NTSTATUS Status;
    ULONG Information;
} IO_STATUS_BLOCK, *PIO_STATUS_BLOCK;

/*
 Define driver cancel routine type.
*/

typedef
VOID
(*PDRIVER_CANCEL) (
    IN struct _DEVICE_OBJECT *DeviceObject,
    IN struct _IRP *Irp
    );

typedef struct _IRP {
    CSHORT Type;
    USHORT Size;

    /*
     The following union is used for one of three purposes:
    
       1. This IRP is an associated IRP.  The field is a pointer to a master
           IRP.
    
        2. This is the master IRP.  The field is the count of the number of
           IRPs which must complete (associated IRPs) before the master can
           complete.
    
        3. This operation is being buffered and the field is the address of
           the system space buffer.
    */

    union {
        struct _IRP *MasterIrp;
        LONG IrpCount;
        PVOID SystemBuffer;
    } AssociatedIrp;

    /*
     I/O status - final status of operation.
    */

    IO_STATUS_BLOCK IoStatus;

    struct _IO_STACK_LOCATION *CurrentStackLocation;
    /*
     Cancel - packet has been canceled.
    */

    BOOLEAN Cancel;

    /*
     Cancel Irql - Irql at which the cancel spinlock was acquired.
    */

    KIRQL CancelIrql;

    /*
     CancelRoutine - Used to contain the address of a cancel routine supplied
     by a device driver when the IRP is in a cancelable state.
    */

    PDRIVER_CANCEL CancelRoutine;

} IRP, *PIRP;


/*
 Define stack location control flags
*/

#define SL_PENDING_RETURNED             0x01
#define SL_INVOKE_ON_CANCEL             0x20
#define SL_INVOKE_ON_SUCCESS            0x40
#define SL_INVOKE_ON_ERROR              0x80

/*

 Define completion routine types for use in stack locations in an IRP
*/

typedef
NTSTATUS
(*PIO_COMPLETION_ROUTINE) (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );

/*
 Define I/O Request Packet (IRP) stack locations
*/

typedef struct _IO_STACK_LOCATION {
    UCHAR MajorFunction;
    UCHAR MinorFunction;
    UCHAR Flags;
    UCHAR Control;

    /*
     The following user parameters are based on the service that is being
     invoked.  Drivers and file systems can determine which set to use based
     on the above major and minor function codes.
    */

    union {

        /*
         System service parameters for:  NtReadFile
        */

        struct {
            ULONG Length;
            ULONG Key;
            LARGE_INTEGER ByteOffset;
        } Read;

        /*
         System service parameters for:  NtWriteFile
        */

        struct {
            ULONG Length;
            ULONG Key;
            LARGE_INTEGER ByteOffset;
        } Write;


        /*
         System service parameters for:  NtDeviceIoControlFile
        
         Note that the user's output buffer is stored in the UserBuffer field
         and the user's input buffer is stored in the SystemBuffer field.
        */

        struct {
            ULONG OutputBufferLength;
            ULONG InputBufferLength;
            ULONG IoControlCode;
/*            PVOID Type3InputBuffer;*/
        } DeviceIoControl;


        /*
         Others - driver-specific
        */

        struct {
            PVOID Argument1;
            PVOID Argument2;
            PVOID Argument3;
            PVOID Argument4;
        } Others;

    } Parameters;

    /*
     Save a pointer to this device driver's device object for this request
     so it can be passed to the completion routine if needed.
    */

    PDEVICE_OBJECT DeviceObject;

    /*
     The following routine is invoked depending on the flags in the above
     flags field.
    */

    PIO_COMPLETION_ROUTINE CompletionRoutine;

    /*
     The following is used to store the address of the context parameter
     that should be passed to the CompletionRoutine.
    */

    PVOID Context;

} IO_STACK_LOCATION, *PIO_STACK_LOCATION;

PIO_STACK_LOCATION 
IoGetNextIrpStackLocation(
        IN PIRP  Irp
        );

#define NTHALAPI
#define NTKERNELAPI
#define NTSYSAPI
#define NTAPI
#define UNALIGNED

/*
 Define the I/O version of a DPC routine.
*/

typedef
VOID
(*PIO_DPC_ROUTINE) (
    IN PKDPC Dpc,
    IN struct _DEVICE_OBJECT *DeviceObject,
    IN struct _IRP *Irp,
    IN PVOID Context
    );


/*
 Define driver reinitialization routine type.
*/

typedef
VOID
(*PDRIVER_REINITIALIZE) (
    IN struct _DRIVER_OBJECT *DriverObject,
    IN PVOID Context,
    IN ULONG Count
    );

NTKERNELAPI
NTSTATUS
IoCreateDevice(
    IN PDRIVER_OBJECT DriverObject,
    IN ULONG DeviceExtensionSize,
    IN PUNICODE_STRING DeviceName OPTIONAL,
    IN DEVICE_TYPE DeviceType,
    IN ULONG DeviceCharacteristics,
    IN BOOLEAN Exclusive,
    OUT PDEVICE_OBJECT *DeviceObject
    );

NTSTATUS 
IoAttachDevice(
        IN PDEVICE_OBJECT  SourceDevice,
        IN PUNICODE_STRING  TargetDevice,
        OUT PDEVICE_OBJECT  *AttachedDevice
        );

VOID 
IoDetachDevice(
        IN OUT PDEVICE_OBJECT  TargetDevice
        );

VOID 
IoSetCompletionRoutine(
        IN PIRP  Irp,
        IN PIO_COMPLETION_ROUTINE  CompletionRoutine,
        IN PVOID  Context,
        IN BOOLEAN  InvokeOnSuccess,
        IN BOOLEAN  InvokeOnError,
        IN BOOLEAN  InvokeOnCancel
                );

NTSTATUS 
IoCallDriver(
        IN PDEVICE_OBJECT  DeviceObject,
        IN OUT PIRP  Irp
        );

PIRP
AllocateIRP(
    );

NTSTATUS
IoCreateSymbolicLink(
        IN PUNICODE_STRING  SymbolicLinkName,
        IN PUNICODE_STRING  DeviceName
	);

NTSTATUS
IoDeleteSymbolicLink(
        IN PUNICODE_STRING  SymbolicLinkName
        );

/*
 Pool Allocation routines (in pool.c)
*/

typedef enum _POOL_TYPE {
    NonPagedPool,
    PagedPool,
    NonPagedPoolMustSucceed,
    DontUseThisType,
    NonPagedPoolCacheAligned,
    PagedPoolCacheAligned,
    NonPagedPoolCacheAlignedMustS,
    MaxPoolType
    } POOL_TYPE;

NTKERNELAPI
PVOID
ExAllocatePool(
    IN POOL_TYPE PoolType,
    IN ULONG NumberOfBytes
    );

NTKERNELAPI
VOID
ExFreePool(
    IN PVOID P
    );

NTSYSAPI
LARGE_INTEGER 
RtlConvertLongToLargeInteger(
        IN LONG  SignedInteger
        );

NTSYSAPI
LARGE_INTEGER 
RtlConvertUlongToLargeInteger(
        IN ULONG  UnsignedInteger
        );

LARGE_INTEGER 
RtlExtendedIntegerMultiply(
        IN LARGE_INTEGER  Multiplicand,
        IN LONG  Multiplier
        );

NTSYSAPI
VOID
NTAPI
RtlInitUnicodeString(
    PUNICODE_STRING DestinationString,
    PCWSTR SourceString
    );

NTSYSAPI
NTSTATUS
NTAPI
RtlAppendUnicodeToString (
    PUNICODE_STRING Destination,
    PWSTR Source
    );

NTSTATUS 
RtlIntegerToUnicodeString(
        IN ULONG  Value,
        IN ULONG  Base,                                 /* optional */
        IN OUT PUNICODE_STRING  String
        );

LONG 
RtlCompareUnicodeString(
        IN PUNICODE_STRING  String1,
        IN PUNICODE_STRING  String2,
        IN BOOLEAN  CaseInSensitive
        );

NTSYSAPI
NTSTATUS
NTAPI
RtlWriteRegistryValue(
    IN ULONG RelativeTo,
    IN PWSTR Path,
    IN PWSTR ValueName,
    IN ULONG ValueType,
    IN PVOID ValueData,
    IN ULONG ValueLength
    );

/*
 Define I/O Driver error log packet structure.  This structure is filled in
 by the driver.
*/

typedef struct _IO_ERROR_LOG_PACKET {
    UCHAR MajorFunctionCode;
    UCHAR RetryCount;
    USHORT DumpDataSize;
    USHORT NumberOfStrings;
    USHORT StringOffset;
    USHORT EventCategory;
    NTSTATUS ErrorCode;
    ULONG UniqueErrorValue;
    NTSTATUS FinalStatus;
    ULONG SequenceNumber;
    ULONG IoControlCode;
    LARGE_INTEGER DeviceOffset;
    ULONG DumpData[1];
}IO_ERROR_LOG_PACKET, *PIO_ERROR_LOG_PACKET;

NTKERNELAPI
PVOID
IoAllocateErrorLogEntry(
    IN PVOID IoObject,
    IN UCHAR EntrySize
    );

NTKERNELAPI
VOID
IoWriteErrorLogEntry(
    IN PVOID ElEntry
    );

NTHALAPI
BOOLEAN 
HalTranslateBusAddress(
	IN INTERFACE_TYPE  InterfaceType,
	IN ULONG  BusNumber,
	IN PHYSICAL_ADDRESS  BusAddress,
	IN OUT PULONG  AddressSpace,
	OUT PPHYSICAL_ADDRESS  TranslatedAddress
	);

NTHALAPI
ULONG
HalGetBusData(
    IN BUS_DATA_TYPE BusDataType,
    IN ULONG BusNumber,
    IN ULONG SlotNumber,
    IN PVOID Buffer,
    IN ULONG Length
    );

NTHALAPI
ULONG
HalGetBusDataByOffset(
    IN BUS_DATA_TYPE BusDataType,
    IN ULONG BusNumber,
    IN ULONG SlotNumber,
    IN PVOID Buffer,
    IN ULONG Offset,
    IN ULONG Length
    );

NTHALAPI
ULONG
HalSetBusData(
    IN BUS_DATA_TYPE BusDataType,
    IN ULONG BusNumber,
    IN ULONG SlotNumber,
    IN PVOID Buffer,
    IN ULONG Length
    );

NTHALAPI
ULONG
HalSetBusDataByOffset(
    IN BUS_DATA_TYPE BusDataType,
    IN ULONG BusNumber,
    IN ULONG SlotNumber,
    IN PVOID Buffer,
    IN ULONG Offset,
    IN ULONG Length
    );

NTKERNELAPI
VOID 
KeStallExecutionProcessor(
	IN ULONG  MicroSeconds
	);

NTKERNELAPI
VOID
KeQuerySystemTime (
    OUT PLARGE_INTEGER CurrentTime
    );

NTSYSAPI
VOID
NTAPI
RtlTimeToTimeFields (
    PLARGE_INTEGER Time,
    PTIME_FIELDS TimeFields
    );

/*
 DPC object
*/

NTKERNELAPI
VOID
KeInitializeDpc (
    IN PKDPC Dpc,
    IN PKDEFERRED_ROUTINE DeferredRoutine,
    IN PVOID DeferredContext
    );

NTKERNELAPI
VOID
KeInitializeSpinLock (
    IN PKSPIN_LOCK SpinLock
    );

NTKERNELAPI
VOID 
KeAcquireSpinLock(
	IN PKSPIN_LOCK  SpinLock,
	OUT PKIRQL  OldIrql
	);

NTKERNELAPI
VOID 
KeReleaseSpinLock(
	IN PKSPIN_LOCK  SpinLock,
	IN KIRQL  NewIrql
	);

NTKERNELAPI
VOID
KeAcquireSpinLockAtDpcLevel (
    IN PKSPIN_LOCK SpinLock
    );

NTKERNELAPI
VOID
KeReleaseSpinLockFromDpcLevel (
    IN PKSPIN_LOCK SpinLock
    );

NTKERNELAPI
VOID
KeInitializeTimer (
    IN PKTIMER Timer
    );

NTKERNELAPI
BOOLEAN
KeCancelTimer (
    IN PKTIMER
    );

NTKERNELAPI
BOOLEAN
KeReadStateTimer (
    PKTIMER Timer
    );

VOID
KeInitializeEvent(
        IN PKEVENT  Event,
        IN EVENT_TYPE  Type,
        IN BOOLEAN  State
        );

LONG
KeSetEvent(
        IN PKEVENT  Event,
        IN KPRIORITY  Increment,
        IN BOOLEAN  Wait
        );

VOID
KeClearEvent(
        IN PKEVENT Event
        );

LONG
KeResetEvent(
        IN PKEVENT  Event
        );

LONG
KeReadStateEvent(
        IN PKEVENT  Event
        );

NTSTATUS
KeWaitForSingleObject(
	IN PVOID  Object,
	IN KWAIT_REASON  WaitReason,
	IN KPROCESSOR_MODE  WaitMode,
	IN BOOLEAN  Alertable,
	IN PLARGE_INTEGER  Timeout                      /* optional */
	);

PIRP 
IoAllocateIrp(
        IN CCHAR  StackSize,
        IN BOOLEAN  ChargeQuota
        );

VOID
IoInitializeIrp(
        IN OUT PIRP  Irp,
        IN USHORT  PacketSize,
        IN CCHAR  StackSize
        );

USHORT
IoSizeOfIrp(
        IN CCHAR  StackSize
        );

VOID
IoFreeIrp(
        IN PIRP  Irp
        );

PIO_STACK_LOCATION
IoGetCurrentIrpStackLocation(
	IN PIRP  Irp
	);

VOID
IoMarkIrpPending(
	IN OUT PIRP  Irp
	);

VOID
IoReleaseCancelSpinLock(
	IN KIRQL  Irql
	);

VOID
IoAcquireCancelSpinLock(
	OUT PKIRQL  Irql
	);

VOID
IoSetCancelRoutine(
	IN PIRP  Irp,
	IN PDRIVER_CANCEL  CancelRoutine
	);

BOOLEAN
IoCancelIrp(
        IN PIRP  Irp
        );

NTKERNELAPI
NTSTATUS
IoRegisterShutdownNotification(
    IN PDEVICE_OBJECT DeviceObject
    );

NTKERNELAPI
BOOLEAN
KeSetTimer (
    IN PKTIMER Timer,
    IN LARGE_INTEGER DueTime,
    IN PKDPC Dpc OPTIONAL
    );

NTKERNELAPI
VOID 
IoDeleteDevice(
	IN PDEVICE_OBJECT  DeviceObject
	);

NTKERNELAPI
NTSTATUS 
IoInitializeTimer(
	IN PDEVICE_OBJECT  DeviceObject,
	IN PIO_TIMER_ROUTINE  TimerRoutine,
	IN PVOID  Context
	);
  
NTKERNELAPI
VOID 
IoStartTimer(
	IN PDEVICE_OBJECT  DeviceObject
	);
  
VOID 
IoStopTimer(
	IN PDEVICE_OBJECT  DeviceObject
	);
  
NTKERNELAPI
VOID
IoStartNextPacket(
    IN PDEVICE_OBJECT DeviceObject,
    IN BOOLEAN Cancelable
    );

NTKERNELAPI
VOID
IoStartPacket(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PULONG Key OPTIONAL,
    IN PDRIVER_CANCEL CancelFunction OPTIONAL
    );

NTKERNELAPI
VOID
IoCompleteRequest(
    IN PIRP Irp,
    IN CCHAR PriorityBoost
    );

UCHAR 
READ_PORT_UCHAR(
	IN PUCHAR  Port
	);

ULONG 
READ_PORT_ULONG(
	IN PULONG  Port
	);

USHORT 
READ_PORT_USHORT(
	IN PUSHORT  Port
	);

VOID 
WRITE_PORT_UCHAR(
	IN PUCHAR  Port,
	IN UCHAR  Value
	);
VOID 
WRITE_PORT_ULONG(
	IN PULONG  Port,
	IN ULONG  Value
	);
  
VOID 
WRITE_PORT_USHORT(
	IN PUSHORT  Port,
	IN USHORT  Value
	);

PVOID 
MmAllocateNonCachedMemory(
	IN ULONG  NumberOfBytes
	);

VOID 
MmFreeNonCachedMemory(
	IN PVOID  BaseAddress,
	IN ULONG  NumberOfBytes
	);
  
VOID 
RtlZeroMemory(
	IN PVOID  Destination,
	IN ULONG  Length
	);
  
ULONG 
RtlCompareMemory(
	IN PVOID  Source1,
	IN PVOID  Source2,
	IN ULONG  Length
	);

VOID 
RtlCopyMemory(
	IN VOID UNALIGNED  *Destination,
	IN CONST VOID UNALIGNED  *Source,
	IN ULONG  Length
	);

VOID 
RtlMoveMemory(
        IN PVOID  Destination,
        IN CONST VOID  *Source,
        IN ULONG  Length
        );

VOID
RtlCopyBytes(
        IN PVOID  Destination,
        IN CONST VOID  *Source,
        IN ULONG  Length
        );

PVOID 
MmMapIoSpace(
	IN PHYSICAL_ADDRESS  PhysicalAddress,
	IN ULONG  NumberOfBytes,
	IN BOOLEAN  CacheEnable
	);
  
VOID 
MmUnmapIoSpace(
	IN PVOID  BaseAddress,
	IN ULONG  NumberOfBytes
	);

#endif //ifndef Linux
#endif //_NTDDK_H
