
#ifndef _SMBDEVEX_H
#define _SMBDEVEX_H

#ifndef UWSVR4
#if (defined(__x86_64__) || defined(__ia64__))
        /* typedef unsigned int ulong_t;  * DWORD is 4 bytes */
#endif
   typedef unsigned long DWORD;
#endif // UWSVR4

#define SMB_READ_Q_FREE_SIZE	3
#define SMB_SEND_Q_FREE_SIZE	3

typedef enum {
NormalPriority,
HighPriority
} SEND_PRIORITY;

typedef enum {
Startup,
Down,
InitFromDown,
Normal
} SMB_STATE;

typedef enum {
Closed,
Opened,
Unload
} SMB_DRIVER_STATE;

typedef enum {
Idle,
WaitForStart,
WaitForSend,
WaitForAck,
WaitForReply,
Completing
} SEND_STATE;

typedef enum {
AgentReply,
Watchdog,
LocalEvent,
Read,
Free,
NoAllocate
} ITEM_TYPE;

typedef struct QItem {
struct QItem *NextItem;
BOOLEAN FreeOnCompletion;
ULONG RetryCount;
SEND_STATE State;
ITEM_TYPE Type;
PIRP Irp;
UCHAR Buffer[MAX_BCL_BUFFER];
} Q_ITEM, *PQ_ITEM;

typedef struct queue {
PQ_ITEM QHead;
PQ_ITEM *QTail;
PQ_ITEM QFree;
} QUEUE,*PQUEUE;


typedef struct SmbDeviceExtension {

// Device specific variables

PDEVICE_OBJECT SmbDeviceObject;
PDEVICE_OBJECT I2cDeviceObject;
SMB_DRIVER_STATE DriverState;
BOOLEAN ReStartSend;

// These are the state variables and Flags

UCHAR LastAckedSequence;
UCHAR NextAvailSequence;

// These are the Event Handling specific variables

KSPIN_LOCK SmbReadLock;
PIRP ReadIrp;
#ifndef UWSVR4
KEVENT SmbReadIrpFreedEvent;
#endif // !UWSVR4
QUEUE ReadQueue;
PQ_ITEM ActiveReadItem;

// These are the SMB Transmit variables

KSPIN_LOCK SmbSendLock;
QUEUE SendQueue;
PQ_ITEM ActiveSendItem;

// These are the SMB watchdog timer variables

LONG SmbStartWatchdogTimer;
LONG SmbHeartbeatInterval;
BOOLEAN SmbWatchdogPending;
BOOLEAN SmbDisableWatchdogFlag;

// These are the Timer varables used for the base timer

LONG SmbAgentReadTimer;
LONG SmbAgentTimeoutCounter;
LONG SmbAgentReadInterval;
KDPC SmbBaseTimerDpcObject;
KTIMER SmbBaseTimerObject;
LARGE_INTEGER SmbBaseTimerPeriod;

// These are the Timer varables used for the send timer

KDPC SmbSendTimerDpcObject;
KTIMER SmbSendTimerObject;
LARGE_INTEGER SmbAckTimeout;
LARGE_INTEGER SmbReplyTimeout;

// State of SMB

SMB_STATE SmbState;

// Static Storage for the Ack BCL Message

PI2C_WRITE_BUF      AckI2cWriteBuffer;

// These are used to keep track of the outstanding read Irp to the I2C 

PIRP I2cReadIrp;
#ifdef START_I2C_READ_FROM_DPC
KDPC SmbSendReadToI2cDpcObject;
#endif // START_I2C_READ_FROM_DPC

// These are used to monitor PowerFail
BOOLEAN AcLossReported;
DWORD AcLossTimer;

UCHAR	SendSetHbBuffer[sizeof(I2C_WRITE_BUF) + sizeof(bcl_cmd_data_t)];

} SMB_DEVICE_EXTENSION, *PSMB_DEVICE_EXTENSION;

// Other typedefs requiring definition



// Value to increment the priority of the originating irp thread on io completion
// Make if the same as that used fro serial IO, currently 2.
#ifdef IO_SERIAL_INCREMENT
#define SMB_INCREMENT	IO_SERIAL_INCREMENT
#else
#define SMB_INCREMENT	IO_NO_INCREMENT
#endif // IO_SERIAL_INCREMENT


// Stuff used to convert between 100ns since 1-1-1601 to secs since 1-1-1970

#define SECS_PER_MIN		60
#define MINS_PER_HOUR		60
#define HOURS_PER_DAY		24
#define SECS_PER_DAY		(SECS_PER_MIN * MINS_PER_HOUR * HOURS_PER_DAY)
#define DAYS_PER_NON_LYEAR	365
#define YEARS_PER_LYEAR		4

#define UNIX_EPOCH_YEAR		1970
#define NT_EPOCH_YEAR		1601

#define DAYS_DIFF_NT_UNIX	(((UNIX_EPOCH_YEAR - NT_EPOCH_YEAR) * DAYS_PER_NON_LYEAR) + \
				 (((1 + UNIX_EPOCH_YEAR - NT_EPOCH_YEAR) / YEARS_PER_LYEAR)) - 3)

#define SECS_DIFF_NT_UNIX	(DAYS_DIFF_NT_UNIX * SECS_PER_DAY)

#define _100NS_PER_SEC		10000000



#ifndef	FIELD_OFFSET
#define FIELD_OFFSET(struct, field) 		((int) &(((struct *)(0))->field))
#endif // FIELD_OFFSET

#define SMB_FREE_QUEUE_ITEM(Queue, QItem)		\
		do {							\
			if(QItem->FreeOnCompletion != TRUE) {		\
                        	QItem->Type = Free;			\
				QItem->NextItem = Queue.QFree;		\
				Queue.QFree = QItem;			\
			}						\
			else { 						\
				ExFreePool(QItem);			\
			}						\
		} while(0)

#define SMB_QUEUE_QUEUE_ITEM(Queue, QItem)		\
                do {							\
			*(Queue.QTail) = QItem; 			\
			Queue.QTail = &QItem->NextItem;		\
		} while(0)


#define SMB_REQUEUE_QUEUE_ITEM(Queue, QItem)	\
		do {							\
	                if(Queue.QHead == NULL) {			\
		                Queue.QTail = &QItem->NextItem;	\
			}						\
			QItem->NextItem = Queue.QHead;			\
                	Queue.QHead = QItem;				\
		} while(0)
		
#define SMB_INITIALIZE_QUEUE(Queue)			\
		do {							\
			Queue.QHead = NULL;				\
			Queue.QTail = &(Queue.QHead);			\
			Queue.QFree = NULL;				\
		} while(0)

#define SMB_TRANSFER_QUEUE(ToQueue, FromQueue)	\
		do {							\
			ToQueue.QHead = FromQueue.QHead;		\
			if(ToQueue.QHead == NULL) {			\
				ToQueue.QTail = &ToQueue.QHead;		\
			}						\
			else {						\
				ToQueue.QTail = FromQueue.QTail;	\
			}						\
		} while(0)

#ifdef INKERNEL
#define DbdPrint 	if(tracepri) tracev
#endif // INKERNEL

#if DBG
#define Leader_0 ""
#define Leader_1 "\n         "
#define Leader_2 "      "
#define Leader_3 "    "
#define Leader_4 "  "
#define Leader_5 ""
#define Leader_6 ""
#define Leader_7 ""
#define Leader_8 ""

#define SMB_DEBUG_PRINT(Level, ArgList)	\
		do { 							\
			if(SmbDebugLevel >= Level) {			\
                                DbgPrint(Leader_##Level);			\
				DbgPrint ArgList;			\
			}						\
		} while(0)

SMB_HEX_DUMP(DWORD Level, UCHAR *Buffer, DWORD Length);

#else
#define SMB_DEBUG_PRINT(Level, ArgList)
#define SMB_HEX_DUMP(Level, Buffer, Length)
#endif // DBG

#endif // _SMBDRV_H

